import uuid


# Create your models here.
from django.contrib.auth.models import (AbstractBaseUser, BaseUserManager, PermissionsMixin)
from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType

from django.db import models
from rest_framework_simplejwt.tokens import RefreshToken


class Idioma(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    nome = models.CharField(max_length=100, null=True)
    code = models.CharField(max_length=100, null=True)
    STATUS = (
        ('Activo', 'Activo'),
        ('Desativado', 'Desativado')
    )
    estado =models.CharField(max_length=100, default='Activo', null=True, choices=STATUS)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_idioma", "Can list idioma"),
        )

    def __str__(self):
        return  self.code

class Traducao(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    idioma = models.TextField(Idioma=True,blank=True)
    chave = models.TextField(null=True, blank=True)
    traducao = models.TextField(null=True,blank=True)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)

    class Meta:
        permissions = (
            ("list_traducao", "Can List Traducao"),
        )

    def __str__(self):
        return  self.chave

class Input(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    nome = models.CharField(max_length=100, null=True)
    class Meta:
        permissions = (
            ("list_input", "Can list input"),
        ) 

    def __str__(self):
        return self.nome


class String(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    texto = models.TextField(null=True, blank=True)
    class Meta:
        permissions = (
            ("list_string", "Can list string"),
        ) 

    def __str__(self):
        return str(self.texto)
    
class InputString(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    input = models.ForeignKey(Input, on_delete=models.CASCADE)
    string = models.ForeignKey(String, on_delete=models.CASCADE)

    class Meta:
        permissions = (
            ("list_input_string", "Can List Input  String"),
        ) 

    def __str__(self):
        return self.input.nome + ' | '+str(self.string.texto)





class UserManager(BaseUserManager):
    def create_user(self, username, email, password=None,  mobile=None,):
        user = self.model(username=username, email=self.normalize_email(email), mobile=mobile)
        user.set_password(password)
        user.save()
        return user

    def create_superuser(self, username, email, password=None):
        if password is None:
            raise TypeError('Password should not be none')

        user = self.create_user(username, email, password)
        user.is_superuser = True
        user.is_verified = True
        user.is_staff = True
        user.save()
        return user


AUTH_PROVIDERS = {'facebook': 'facebook', 'google': 'google','twitter': 'twitter', 'email': 'email'}


def profile_image_path(instance, file_name):
    return f'images/users/{instance.id}/{file_name}'

class User(AbstractBaseUser, PermissionsMixin):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    perfil = models.ImageField(default='user.png', upload_to=profile_image_path, null=True, blank=True)
    username = models.CharField(max_length=255, unique=False)
    mobile = models.CharField(max_length=55,  null=True, unique=True, blank=True, default=None)
    is_verified_mobile = models.BooleanField(blank=False, default=False)
    counter = models.IntegerField(default=0, blank=False)   # For HOTP Verification
    email = models.EmailField(max_length=255, null=True, unique=True, blank=True, default=None)
    language = models.CharField(max_length=25,default="PT-PT")
    is_verified = models.BooleanField(default=False)
    is_active = models.BooleanField(default=True)
    is_staff = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    date_joined = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    auth_provider = models.CharField(max_length=255, blank=False,null=False, default=AUTH_PROVIDERS.get('email'))

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['username']

    objects = UserManager()
    
    def save(self, *args, **kwargs):
        if self.email == '':
            self.email = None
        if self.mobile =='':
            self.mobile = None
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = "User"
        verbose_name_plural = "Users"
        permissions = (
            ("list_user", "Can list user"),
        )

    def __str__(self):
        return self.username

    def tokens(self):
        refresh =  RefreshToken.for_user(self) 
        return {
            'refresh': str(refresh),
            'access': str(refresh.access_token)
        }




class UserLogin(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    dispositivo = models.TextField(null=True)
    mobile = models.CharField(max_length=100,null=True)
    info = models.TextField(null=True)
    local_lat = models.CharField(max_length=100, null=True)
    local_lon = models.CharField(max_length=100, null=True)
    local_nome = models.CharField(max_length=100, null=True)
    data = models.DateField(null=True, auto_now_add=True)
    hora = models.TimeField(null=True, auto_now_add=True)
    is_blocked = models.BooleanField(default= False)

    class Meta:
        permissions = (
            ("list_userlogin", "Can list user login"),
        )
       
    def __str__(self):
        return  str(self.dispositivo)


def icon_path(instance, file_name):
    return f'{instance.nome}/{file_name}'
class TipoEntidade(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    nome = models.CharField(max_length=100, null=True)
    icon = models.FileField(upload_to=icon_path, default='logo.png', blank=True)
    license = models.TextField(default='license')
    label = models.CharField(max_length=100, null=True)
    header= models.TextField(default='bg-grey-2 text-grey-9')
    menuEsquerdo= models.TextField(default='bg-grey-2 text-grey-9')
    footer = models.TextField(default='bg-grey-2 text-grey-9')
    ordem = models.IntegerField(default=2)
    crair_entidade = models.BooleanField(max_length=100, null=True, default=True)

    STATUS = (
        ('Activo', 'Activo'),
        ('Desativado', 'Desativado')
    )

    estado = models.CharField(max_length=100, null=True, choices=STATUS)
    groups = models.ManyToManyField(Group, blank=True)
    modelos = models.ManyToManyField(ContentType, blank=True)
    language = models.CharField(max_length=25, default="PT")
    link= models.CharField(max_length=500, default="link")
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_tipoentidade", "Can list tipo entidade"),
        )
    def __str__(self):
        return self.nome


class Modulo(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    nome = models.CharField(max_length=100, null=True)
    STATUS = (
        ('Activo', 'Activo'),
        ('Desativado', 'Desativado')
    )
    estado = models.CharField(max_length=100, null=True, choices=STATUS)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_modulo", "Can list modulo"),
        )

    def __str__(self):
        return  self.nome

class FrontEnd(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    nome = models.CharField(max_length=100, null=True)
    fek = models.CharField(max_length=300, null=True)
    fep = models.CharField(max_length=300, null=True)
    STATUS = (
        ('Read', 'Read'),
        ('Write', 'write'),
        ('ReadWrite','ReadWrite')
    )
    rule = models.CharField(max_length=100, null=True, choices=STATUS)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.TimeField(null=True, auto_now_add=True)
    updated_at_time = models.DateTimeField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_front_end", "Can list front end"),
        )

    def __str__(self):
        return  self.nome

def logo_path(instance, file_name):
    return f'{instance.tipo_entidade.nome}/{instance.nome}/{file_name}'
class Entidade(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    nome = models.CharField(max_length=100, null=True, default='-')
    logo = models.FileField(upload_to=logo_path, default='logo.png', blank=True)
    display_logo = models.BooleanField(default=True, null=True, blank=True)
    display_bar = models.BooleanField(default=True, null=True, blank=True)
    display_qr = models.BooleanField(default=True, null=True, blank=True)
    tipo_entidade = models.ForeignKey(TipoEntidade, on_delete=models.CASCADE)
    entidade_bancaria = models.CharField(max_length=100, null=True, blank=True)
    groups = models.ManyToManyField(Group, blank=True)
    modelos = models.ManyToManyField(ContentType, blank=True)
    admins = models.ManyToManyField(User, blank=False)
    ipu = models.CharField(max_length=500, null=True, blank=True)
    rodape = models.CharField(max_length=2000, null=True)
    disc_space = models.FloatField( default=1048576.0, null=True)
    disc_used_space = models.FloatField(default=0.0, null=True)
    disc_free_space = models.FloatField(default=1048576.0, null=True)

    STATUS = (
        ('Activo', 'Activo'),
        ('Desativado', 'Desativado')
    )
    estado =models.CharField(max_length=100, null=True, default='Desativado', choices=STATUS)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)


    def save(self, *args, **kwargs):
        if self.disc_free_space is None or self.disc_free_space > self.disc_space:
            self.disc_free_space = self.disc_space - self.disc_used_space
        super(Entidade, self).save(*args, **kwargs)

    class Meta:
        permissions = (
            ("list_entidade", "Can list entidade"),
        )

    def __str__(self):
        return  self.nome

    
class Sucursal(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    nome = models.CharField(max_length=100, null=True)
    entidade = models.ForeignKey(Entidade, on_delete=models.CASCADE)
    l_latitude= models.CharField(max_length=100, default='.', null=True)
    l_longetude= models.CharField(max_length=100,  default='.', null=True)
    endereco = models.CharField(max_length=300,  default='.', null=True)
    rodape = models.CharField(max_length=600,  default='.', null=True)
    icon = models.CharField(max_length=100,  default='.', null=True)
    label = models.CharField(max_length=100,  default='.', null=True)
    groups = models.ManyToManyField(Group,  default='.', blank=True)
    STATUS = (
        ('Activo', 'Activo'),
        ('Desativado', 'Desativado')
    )
    estado =models.CharField(max_length=100, default='Desativado', null=True, choices=STATUS)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_sucursal", "Can list sucursal"),
        )

    def __str__(self):
        return  self.nome



class EntidadeGroup(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    entidade = models.ForeignKey(Entidade, on_delete=models.CASCADE)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_entidadegroup", "Can list entidade group"),
        )
    def __str__(self):
        return str(self.entidade.nome)+ "  |  "+str(self.group.name)


class EntidadeUser(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    entidade = models.ForeignKey(Entidade, on_delete=models.CASCADE)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_entidadeuser", "Can list entidade user"),
        )

    def __str__(self):
        return str(self.entidade.nome) + "  |  " + str(self.user.username)


class SucursalGroup(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    sucursal = models.ForeignKey(Sucursal, on_delete=models.CASCADE)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_sucursalgroup", "Can list sucursal group"),
        )
    def __str__(self):
        return str(self.sucursal.nome)+" | "+str(self.group.name)+" "

class SucursalUser(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    sucursal = models.ForeignKey(Sucursal, on_delete=models.CASCADE)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_sucursaluser", "Can list sucursal user"),
        )

    def __str__(self):
        return str(self.sucursal.nome)+" | "+str(self.user.username)+" "


class SucursalUserGroup(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    sucursal = models.ForeignKey(Sucursal, on_delete=models.CASCADE)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_sucursalusergroup", "Can list sucursal user group"),
        )

    def __str__(self):
        return str(self.user.username)+ "   |  "+str(self.sucursal.nome)+ "   |  "+str(self.group.name)


class EntidadeModulo(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    entidade = models.ForeignKey(Entidade, on_delete=models.CASCADE)
    modulo = models.ForeignKey(Modulo, on_delete=models.CASCADE)
    STATUS = (
        ('Activo', 'Activo'),
        ('Desativado', 'Desativado')
    )
    estado = models.CharField(max_length=100, null=True, choices=STATUS)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_entidademodulo", "Can list entidade modulo"),
        )

    def __str__(self):
        return str(self.entidade.nome) + "  |  " + str(self.modulo.nome)


class TipoEntidadeModulo(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    tipo_entidade = models.ForeignKey(TipoEntidade, on_delete=models.CASCADE)
    modulo = models.ForeignKey(Modulo, on_delete=models.CASCADE)
    STATUS = (
        ('Activo', 'Activo'),
        ('Desativado', 'Desativado')
    )
    estado = models.CharField(max_length=100, null=True, choices=STATUS)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_tipoentidademodulo", "Can list tipo entidade modulo"),
        )

    def __str__(self):
        return str(self.tipo_entidade.nome) + "  |  " + str(self.modulo.nome)


class Ficheiro(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    ficheiro = models.FileField( upload_to='ficheiros', null=True, blank=True)
    size = models.FloatField(null=False)
    modelo = models.CharField(max_length=100, null=True)    
    STATUS = (
        ('Activo', 'Activo'),
        ('Desativado', 'Desativado')
    )
    estado =models.CharField(max_length=100, null=True, default='Desactivado', choices=STATUS)

    ESCOLHA = (
        ('File', 'File'),
        ('Perfil', 'Perfil'),
        ('Logo', 'Logo'),
        ('Foto', 'Foto'),
        ('CapaSite', 'CapaSite')
    )
    
    funcionalidade =models.CharField(max_length=100, null=True, default='File', choices=ESCOLHA)
    sucursal = models.ForeignKey(Sucursal, on_delete=models.CASCADE, null=True, blank=True)
    entidade = models.ForeignKey(Entidade, on_delete=models.CASCADE, null=True, blank=True)
    chamador = models.CharField(max_length= 100, null=True, blank=True)
    created_at = models.DateField(null=True, auto_now_add=True)
    updated_at = models.DateField(null=True, auto_now=True)
    created_at_time = models.DateField(null=True, auto_now_add=True)
    updated_at_time = models.DateField(null=True, auto_now=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        permissions = (
            ("list_ficheiro", "Can list ficheiros"),
        )

    def __str__(self):
        return  self.ficheiro.name
      


from django.shortcuts import render
from django.template import loader
from rest_framework import generics, status, views, permissions
from django.core.exceptions import ObjectDoesNotExist
from django.contrib import auth
import json
from django.core.serializers.json import DjangoJSONEncoder
from django.contrib.auth.models import Group

from .serializers import *
from rest_framework.response import Response
from rest_framework_simplejwt.tokens import RefreshToken
from .models import User, UserLogin, TipoEntidade

from django.core.mail import send_mail


# import environ
# from core.SMS import SMS

# Initialise environment variables
# env = environ.Env()
from django.urls import reverse
import jwt
from drf_yasg import openapi
from django.conf import settings

# from .renderers import UserRenderer
from django.contrib.auth.tokens import PasswordResetTokenGenerator
from django.utils.encoding import smart_str, force_str, smart_bytes, DjangoUnicodeDecodeError
from django.utils.http import urlsafe_base64_decode, urlsafe_base64_encode
from django.contrib.sites.shortcuts import get_current_site
from django.urls import reverse

from django.shortcuts import redirect
from django.http import HttpResponsePermanentRedirect
import os
from django.core.mail import EmailMultiAlternatives

import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart

from .classes import UserName
import re


def is_valid_email(email):
    regex = r'^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'
    return re.match(regex, email) is not None

from datetime import datetime

from rest_framework.views import APIView

from .models import User
import base64
import pyotp


# This class returns the string needed to generate the key
class generateKey:
    @staticmethod
    def returnValue(phone):
        print(settings.OTP_KEY)
        print(phone)
        return str(phone) + str(datetime.date(datetime.now())) + settings.OTP_KEY


class getPhoneNumberRegistered(APIView):
    # Get to Create a call for OTP
    @staticmethod
    def post(request):
        phone =  str('+' + request.data["mobile"]).replace('+', '')
        try:
            user = User.objects.get(mobile=phone)  # if mobile already exists the take this else create New One
        except ObjectDoesNotExist:
            if 'reset_senha' in request.query_params:
                return Response({'alert_error': 'O numero nao existe <br><b>' + phone + '</b>'}, status=400)  # Just for demonstration
                
            nome = str(str(request.META['HTTP_ORIGIN']).split('.')[0].upper()).split('/')[-1]
            request.data['username'] = UserName.Create(str(request.data['username'].replace(' ', '_')))
            request.data['mobile'] = phone
            request.data['email'] = None
            serializer = RegisterSerializer(data=request.data)
            serializer.is_valid(raise_exception=True)
            serializer.save()
            user_data = serializer.data
            user = User.objects.get(mobile=phone)
            tipoEntidade = TipoEntidade.objects.get(nome=str(nome).capitalize())

            pessoa = Pessoa()
            pessoa.user= user
            pessoa.save()
            
        user.counter += 1  # Update Counter At every Call
        user.save()
        keygen = generateKey()
        key = base64.b32encode(keygen.returnValue(phone).encode())  # Key is generated
        OTP = pyotp.HOTP(key)  # HOTP Model for OTP is created
        try:
            SMS.send(from__ ='+17244014353', to__='+258'+str(phone), text__='Nao partilhe esse codigo:\nOTP '+OTP.at(user.counter))
        except Exception as e:
            return Response({'alert_error': 'Erro<br>'+ str(e) }, status=400)  # Just for demonstration
            
        return Response({'alert_success': 'Enviamos um OTP para o seu numero <br><b>' + phone + '</b>', "OTP": OTP.at(user.counter)}, status=201)  # Just for demonstration

    # This Method verifies the OTP
    @staticmethod
    def put(request):
        phone = str( '+' + request.data["mobile"]).replace('+', '')
        try:
            user = User.objects.get(mobile=phone)
        except ObjectDoesNotExist:
            return Response("User does not exist", status=404)  # False Call

        keygen = generateKey()
        key = base64.b32encode(keygen.returnValue(phone).encode())  # Generating Key
        OTP = pyotp.HOTP(key)  # HOTP Model
        if OTP.verify(request.data["otp"], user.counter):  # Verifying the OTP
            user.is_verified_mobile = True
            user.save()
            return Response({'alert_success': 'Voce esta Autorizado!', 'id': user.id, 'otp': request.data["otp"], 'mobile': request.data["mobile"]}, status=202)
        return Response({'alert_error': 'OTP is wrong'}, status=400)


# Time after which OTP will expire
EXPIRY_TIME = 120 # seconds

class getPhoneNumberRegistered_TimeBased(APIView):
    # Get to Create a call for OTP
    @staticmethod
    def get(request, phone):
        phone = '+' + str(phone).replace('+', '')
        try:
            mobile = User.objects.get(mobile=phone)  # if mobile already exists the take this else create New One
        except ObjectDoesNotExist:
            User.objects.create(
                mobile=phone,
            )
            mobile = User.objects.get(mobile=phone)  # user Newly created Model
        mobile.save()  # Save the data
        keygen = generateKey()
        key = base64.b32encode(keygen.returnValue(phone).encode())  # Key is generated
        OTP = pyotp.TOTP(key,interval = EXPIRY_TIME)  # TOTP Model for OTP is created
        print(OTP.now())
        # Using Multi-Threading send the OTP Using Messaging Services like Twilio or Fast2sms
        SMS.send(from__='+13192205575', to__=phone, text__='Nao partilhe esse codigo:\nOTP '+OTP.now())
        return Response({"OTP": OTP.now()}, status=200)  # Just for demonstration

    # This Method verifies the OTP
    @staticmethod
    def post(request, phone):
        phone = '+' + str(phone).replace('+', '')
        try:
            mobile = User.objects.get(mobile=phone)
        except ObjectDoesNotExist:
            return Response("User does not exist", status=404)  # False Call

        keygen = generateKey()
        key = base64.b32encode(keygen.returnValue(phone).encode())  # Generating Key
        OTP = pyotp.TOTP(key,interval = EXPIRY_TIME)  # TOTP Model 
        if OTP.verify(request.data["otp"]):  # Verifying the OTP
            mobile.isVerified = True
            mobile.save()
            return Response("You are authorised", status=200)
        return Response("OTP is wrong/expired", status=400)



























class CustomRedirect(HttpResponsePermanentRedirect):
    allowed_schemes = [os.environ.get('APP_SCHEME'), 'http', 'https']


class RegisterView(generics.GenericAPIView):
    serializer_class = RegisterSerializer
    # renderer_classes = (UserRenderer,)

    def post(self, request):
        origin = request.META.get("HTTP_ORIGIN", 'http://mytech.co.mz')
        nome = str(str(origin).split('.')[0].upper()).split('/')[-1]

        data = request.data.copy()
        data['username'] = UserName.Create(str(request.data['username'].replace(' ', '_')))
        data['mobile'] = None
        user = data
        serializer = self.serializer_class(data=user)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        user_data = serializer.data

        user = User.objects.get(email=user_data['email'])
        tipoEntidade = TipoEntidade.objects.get(nome=str(nome).capitalize())
        logo = FullPath.url(None, tipoEntidade.icon.name)
        
        token = RefreshToken.for_user(user).access_token

        current_site = get_current_site(request).domain
        relativeLink = reverse('email_verify')
        print(current_site)
  
    
        
        html_message = loader.render_to_string( 'email_confirmacao.html',
            {
                'username': user.username,
                'token': str(token),
                'entidade': nome,
                'tipoentidade': nome,
                'logo': logo,
                'origen': request.META['HTTP_ORIGIN']
            }
        )

        
        email_subject = 'Bem Vindo '+ nome
        to_list = [user.email]
        
        mail = EmailMultiAlternatives(
            subject=email_subject,
            body='Clique no link para restaurar sua senha',
            to=to_list,
        )
        mail.attach_alternative(html_message, "text/html")
          
        try:
            mail.send()
        except Exception as e:
            print(e, end="\n")
            print("Unable to send mail.")
            # logger.error(e)
            user.delete()
            return Response({'alert_error': str (e)}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
        

        # return Response({'alert_success': 'Enviamos um link para redefinir sua senha'}, status=status.HTTP_200_OK)
        add = {'alert_success': '%-' + user_data['username'] + '-% foi criado com sucesso'}
        data = json.loads(json.dumps(user_data, cls=DjangoJSONEncoder))
        data.update(add)
        return Response(data, status=status.HTTP_201_CREATED)


class VerifyEmail(views.APIView):
    serializer_class = EmailVerificationSerializer
    token_param_config = openapi.Parameter('token', in_=openapi.IN_QUERY, description='Description', type=openapi.TYPE_STRING)
    def get(self, request):
        token = request.GET.get('token')
        try:
            payload = jwt.decode(token, settings.SECRET_KEY, algorithms=["HS256"])
            user = User.objects.get(id=payload['user_id'])
            if not user.is_verified:
                user.is_verified = True
                user.save()
            return Response({'alert_success': 'Ativado com sucesso'}, status=status.HTTP_200_OK)
        except jwt.ExpiredSignatureError as identifier:
            return Response({'alert_error': 'Ativação expirada'}, status=status.HTTP_400_BAD_REQUEST)
        except jwt.exceptions.DecodeError as identifier:
            return Response({'alert_error': 'Token inválido'}, status=status.HTTP_400_BAD_REQUEST)


class LoginAPIView(generics.GenericAPIView):
    serializer_class = LoginSerializer
    def post(self, request):
        if not is_valid_email (request.data['email']):
            request.data['email'] = request.data['email']+'@paravalidar.com'

        serializer = self.serializer_class(data=request.data, context={'request': request})
        serializer.is_valid(raise_exception=True)
        data = request.data.copy()
        data['user'] = serializer.data['id']
        userLogin = UserLoginSerializer(data = data)
        if userLogin.is_valid(raise_exception=True):
            userLogin.save()

        return Response(serializer.data, status=status.HTTP_200_OK)


class MeAPIView(generics.GenericAPIView):
    serializer_class = MeSerializer
    def get(self, request):
        serializer = self.serializer_class(request.user, context={'request': request})
        return Response(serializer.data, status=status.HTTP_200_OK)


class ChangePasswordMobileAPIView(generics.GenericAPIView):
    def post(self, request):
        phone = request.data['mobile']
        user = User.objects.get(mobile=phone)
        keygen = generateKey()
        key = base64.b32encode(keygen.returnValue(phone).encode())  # Generating Key
        OTP = pyotp.HOTP(key)  # HOTP Model
        if OTP.verify(request.data["otp"], user.counter): # Verifying the OTP
            user.set_password(request.data['password'])
            user.save()
            return Response({'alert_success': 'Senha Resetada com sucesso!'}, status=202)
        return Response({'alert_error': 'OTP errado ou experado!'}, status=tatus.HTTP_400_BAD_REQUEST)
        

class ChangePasswordEmailAPIView(generics.GenericAPIView):
    def post(self, request):
        user = auth.authenticate(email=request.data['email'], password=request.data['password'])
        if not user:
            return Response({'alert_error': 'Senha autual esta errada!'}, status=status.HTTP_400_BAD_REQUEST)
        
        if len(request.data['passwordNova']) < 8:
            return Response({'alert_error': 'Senha deve ter no minimo 8 caracteres!'}, status=status.HTTP_400_BAD_REQUEST)
        
        user = User.objects.get(id=user.id)
        user.set_password(request.data['passwordNova'])
        user.save()
        return Response({'alert_success': 'Senha alterada com sucesso!'}, status=202)


class LoginsAPIView(generics.GenericAPIView):
    def get(self, request):
        userLogin = UserLogin.objects.filter(user = request.user).order_by('data', '-hora')
        userLogins = UserLoginSerializer(userLogin, many=True)
        return Response(userLogins.data, status=status.HTTP_200_OK)


class RequestPasswordResetEmail(generics.GenericAPIView):
    serializer_class = ResetPasswordEmailRequestSerializer

    def post(self, request):
        nome = str(str(request.META['HTTP_ORIGIN']).split('.')[0].upper()).split('/')[-1]
       
        serializer = self.serializer_class(data=request.data)

        email = request.data.get('email', '')

        if User.objects.filter(email=email).exists():

            user = User.objects.get(email=email)
            uidb64 = urlsafe_base64_encode(smart_bytes(user.id))
            token = PasswordResetTokenGenerator().make_token(user)
            current_site = get_current_site(request=request).domain
            
            relativeLink = reverse('password-reset-confirm', kwargs={'uidb64': uidb64, 'token': token})
            
            redirect_url = request.data.get('redirect_url', '')
            try:
                absurl = '\n' + request.META['HTTP_ORIGIN'] + '/#/resetpassword' + relativeLink
            except:
                absurl = '\n http:mws.mytech.co.mz/#/resetpassword' + relativeLink

            tipoEntidade = TipoEntidade.objects.get(nome=str(nome).capitalize())
     
            logo = 'logo'
      

            email_body =  absurl+ "?redirect_url=" + redirect_url


            html_message = loader.render_to_string(
                'email_reset.html',
                {
                    'link':email_body,
                    'username':user.username,
                    'logo': logo
                }
            )
            
            email_subject = 'Restaurar senha'
            to_list = [email]
            try:

                mail = EmailMultiAlternatives(
                    subject=email_subject,
                    body='Clique no link para restaurar sua senha',
                    to=to_list,
                )
                mail.attach_alternative(html_message, "text/html")
                mail.send() 
            except Exception as e:
                print(e, end="\n")
                return Response({'alert_error': str (e)}, status=500)

        return Response({'alert_success': 'Enviamos um link para redefinir sua senha'}, status=status.HTTP_200_OK)


class Mail (generics.GenericAPIView):
    serializer_class = ResetPasswordEmailRequestSerializer

    def get(self, request):
        nome = 'email' #str(str(request.META['HTTP_ORIGIN']).split('.')[0].upper()).split('/')[-1]
       
        serializer = self.serializer_class(data=request.data)

        email = request.query_params.get('email', 'metanochava@gmail.com')
    

        if User.objects.filter(email=email).exists():
            user = User.objects.get(email=email)
            uidb64 = urlsafe_base64_encode(smart_bytes(user.id))
            token = PasswordResetTokenGenerator().make_token(user)
            current_site = get_current_site(request=request).domain
            
            relativeLink = reverse('password-reset-confirm', kwargs={'uidb64': uidb64, 'token': token})
            
            redirect_url = request.data.get('redirect_url', '')
            try:
                absurl = '\n' + request.META['HTTP_ORIGIN'] + '/#/resetpassword' + relativeLink
            except:
                absurl = '\n http:mws.mytech.co.mz/#/resetpassword' + relativeLink

            logo = 'logo'

            email_body =  absurl+ "?redirect_url=" + redirect_url

            html_message = loader.render_to_string(
                'email_template_reset.html',
                {
                    'link':email_body,
                    'username':user.username,
                    'logo': logo
                }
            )

            
            email_subject = 'Restaurar senha'
            to_list = [email]
            try:

                mail = EmailMultiAlternatives(
                    subject=email_subject,
                    body='Clique no link para restaurar sua senha',
                    to=to_list,
                )
                mail.attach_alternative(html_message, "text/html")
                mail.send() 
            except Exception as e:
                print(e, end="\n")
                return Response({'error - ': str (e)}, status=500)

            
            try:
                sender_email = "noreplay@mytech.co.mz"
                receiver_email = email
                password = "noreplaygmail"

                # Create the MIME object
                message = MIMEMultipart()
                message["From"] = sender_email
                message["To"] = receiver_email
                message["Subject"] = email_subject

                # Attach the body of the email
                body = html_message
                message.attach(MIMEText(body, "html"))

                with smtplib.SMTP_SSL('whost02.whost.co.mz', 465) as server:
                    server.login(sender_email, password)
                    server.sendmail(sender_email, receiver_email, message.as_string())
                    server.quit()
                    
                
                send_mail(
                    subject='Assunto de Teste',
                    message='Olá, este é um e-mail de teste enviado via Django!',
                    from_email='noreplay@mytech.co.mz',
                    recipient_list=['metanochava@gmail.com'],
                    fail_silently=True,
                )

            except Exception as e:
                print(e, end="\n")
                print("Unable to send mail.")
                return Response({'error': str (e)}, status=500)
                # logger.error(e)
                # logger.error("Unable to send mail.")
                pass

        return Response({'alert_success': 'Enviamos um link para redefinir sua senha'}, status=status.HTTP_200_OK)


class PasswordTokenCheckAPI(generics.GenericAPIView):
    serializer_class = SetNewPasswordSerializer

    def get(self, request, uidb64, token):

        redirect_url = request.GET.get('redirect_url')

        try:
            id = smart_str(urlsafe_base64_decode(uidb64))
            user = User.objects.get(id=id)

            if not PasswordResetTokenGenerator().check_token(user, token):
                if len(redirect_url) > 3:
                    return CustomRedirect(redirect_url+'?token_valid=False')
                else:
                    return CustomRedirect(os.environ.get('FRONTEND_URL', '')+'?token_valid=False')

            if redirect_url and len(redirect_url) > 3:
                return CustomRedirect(redirect_url+'?token_valid=True&message=Credentials Valid&uidb64='+uidb64+'&token='+token)
            else:
                return CustomRedirect(os.environ.get('FRONTEND_URL', '')+'?token_valid=False')

        except DjangoUnicodeDecodeError as identifier:
            try:
                if not PasswordResetTokenGenerator().check_token(user):
                    return CustomRedirect(redirect_url+'?token_valid=False')
                    
            except UnboundLocalError as e:
                return Response({'alert_error': 'O token não é válido, solicite um novo'}, status=status.HTTP_400_BAD_REQUEST)



class SetNewPasswordAPIView(generics.GenericAPIView):
    serializer_class = SetNewPasswordSerializer
    def patch(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        return Response({'alert_success': True, 'message': 'Senha redefinida com sucesso'}, status=status.HTTP_200_OK)


class LogoutAPIView(generics.GenericAPIView):
    serializer_class = LogoutSerializer
    permission_classes = (permissions.IsAuthenticated,)
    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(status=status.HTTP_204_NO_CONTENT)




MENU = "Core"
ICON = "menu"
ROLE = "core"


SUBMENUS = [
    {
        "menu": "Dashboard",
        "icon": "dashboard",
        "role": ROLE + "dashboard",
        "rota": ROLE + "dashboard",
    },
    {
        "menu": "Tipoentidade",
        "icon": "corporate_fare",
        "role": "list_tipoentidade",
        "rota": "list_tipoentidade",
    },    
    {
        "menu": "Entidades",
        "icon": "home",
        "role": "list_entidade",
        "rota": "list_entidade",
    },  
     
    {
        "menu": "Entidade",
        "icon": "home",
        "role": "view_entidade",
        "rota": "view_entidade",
    },  
    {
        "menu": "Sucursal",
        "icon": "house",
        "role": "list_sucursal",
        "rota": "list_sucursal",
    }, 
    {
        "menu": "Perfil",
        "icon": "groups",
        "role": "list_perfil",
        "rota": "list_perfil",
    }, 
    {
        "menu": "Usuario",
        "icon": "face",
        "role": "list_user",
        "rota": "list_usuario",
    },{
        "menu": "UsuarioAll",
        "icon": "people",
        "role": "list_user_all",
        "rota": "list_usuarioall",
    },  
    {
        "menu": "input",
        "icon": "input",
        "role": "list_input",
        "rota": "list_input",
    }, {
        "menu": "Cometario",
        "icon": "mode_comment",
        "role": "list_cometario",
        "rota": "list_cometario",
    }, 
    
]


from ..models import User

def Create(name):
    try:
        username = User.objects.get(username=name).username
        numero = int("".join(filter(str.isdigit, username)))
        letras = "".join(filter(str.isalpha, s))
        h = numero+1
        username =f'{letras}{h}'
        return Create(username)
    except:
        return name


from django.contrib import admin
from django.contrib.auth.models import Permission
from .models import *


admin.site.site_title = 'Auth'
admin.site.index_title = 'Mytech Auth Rest'



class TraducaoAdmin(admin.ModelAdmin):
    list_display_links = ('id',)
    list_display = ['id', 'chave', 'traducao', 'idioma']
admin.site.register(Traducao, TraducaoAdmin)

class EntidadeGroupAdmin(admin.ModelAdmin):
    list_display = ['id', 'entidade', 'group']
    list_display_links = ('id',)
admin.site.register(EntidadeGroup, EntidadeGroupAdmin)

class FicheiroAdmin(admin.ModelAdmin):
    list_display_links = ('id',)
    list_display =['id', 'ficheiro', 'size', 'modelo', 'estado', 'chamador', 'funcionalidade', 'sucursal', 'entidade', 'created_at', 'updated_at', 'created_at_time', 'updated_at_time', 'is_deleted']
admin.site.register(Ficheiro, FicheiroAdmin)

class PermissionAdmin(admin.ModelAdmin):
    list_display = ['id', 'name', 'codename', 'content_type']
    list_display_links = ('id',)
    search_fields = ['id', 'name']
admin.site.register(Permission, PermissionAdmin)

class FrontEndAdmin(admin.ModelAdmin):
    list_display = ['id', 'nome', 'fek', 'fep', 'rule', 'created_at', 'updated_at', 'created_at_time', 'updated_at_time', 'is_deleted']
admin.site.register(FrontEnd, FrontEndAdmin)


class IdiomaAdmin(admin.ModelAdmin):
    list_display = ['id', 'nome',  'code' ]
    list_display_links = ('id', )
    search_fields = ['id',  'nome' ]
admin.site.register(Idioma, IdiomaAdmin)


class TipoEntidadeAdmin(admin.ModelAdmin):
    list_display = ['id', 'nome', 'icon', 'created_at', 'updated_at', 'is_deleted']
    list_display_links = ('id', 'nome',)
    search_fields = ['nome']
admin.site.register(TipoEntidade, TipoEntidadeAdmin)


class UserAdmin(admin.ModelAdmin):
    list_display = ['id', 'username', 'mobile','is_verified_mobile', 'counter', 'email', 'perfil', 'language', 'is_verified', 'is_active', 'is_staff', 'updated_at', 'auth_provider', 'created_at']
    list_display_links = ('id', 'username', 'email')
    search_fields = ['username', 'mobile', 'email', 'nome', 'nome_meio', 'apelido']
admin.site.register(User, UserAdmin)

class UserLoginAdmin(admin.ModelAdmin):
    list_display = ['id', 'user', 'dispositivo', 'info', 'mobile', 'local_nome', 'local_lat', 'local_lon', 'data', 'hora', 'is_blocked']
    list_display_links = ('id', )
    search_fields = ['local_nome', 'dispositivo', 'user' ]
admin.site.register(UserLogin, UserLoginAdmin)

class EntidadeAdmin(admin.ModelAdmin):
    list_display = ['id', 'nome', 'logo', 'ipu', 'admin_list', 'display_logo', 'display_qr', 'display_bar',  'tipo_entidade', 'entidade_bancaria', 'disc_space', 'disc_used_space', 'disc_free_space', 'rodape', 'estado', 'created_at', 'updated_at', 'is_deleted']
    list_display_links = ('id', 'nome',)
    search_fields = ['nome']

    def admin_list(self, obj):
        return ", ".join(
            [u.username for u in obj.admins.all()]
        )
    admin_list.short_description = "admins"
admin.site.register(Entidade, EntidadeAdmin)

class EntidadeUserAdmin(admin.ModelAdmin):
    list_display = ['id', 'user', 'entidade', 'created_at', 'updated_at', 'is_deleted']
    list_display_links = ('id', 'user',)
    search_fields = ['user']
admin.site.register(EntidadeUser, EntidadeUserAdmin)

class SucursalAdmin(admin.ModelAdmin):
    list_display = ['id', 'nome',  'created_at', 'updated_at', 'is_deleted']
    list_display_links = ('id', 'nome',)
    search_fields = ['nome']
admin.site.register(Sucursal, SucursalAdmin)

class SucursalGroupAdmin(admin.ModelAdmin):
    list_display = ['id', 'sucursal', 'group', 'created_at', 'updated_at', 'is_deleted']
    list_display_links = ('id', 'sucursal',  'group',)
    search_fields = ['sucursal', 'group']
admin.site.register(SucursalGroup, SucursalGroupAdmin)

class SucursalUserAdmin(admin.ModelAdmin):
    list_display = ['id', 'sucursal', 'user', 'created_at', 'updated_at', 'is_deleted']
    list_display_links = ('id', 'sucursal',  'user',)
    search_fields = ['sucursal', 'user']
admin.site.register(SucursalUser, SucursalUserAdmin)

class SucursalUserGroupAdmin(admin.ModelAdmin):
    list_display = ['id', 'sucursal', 'user', 'group', 'created_at', 'updated_at', 'is_deleted']
    list_display_links = ('id', 'sucursal',  'user','group', )
    search_fields = ['sucursal', 'user', 'group']
admin.site.register(SucursalUserGroup, SucursalUserGroupAdmin)

class EntidadeModuloAdmin(admin.ModelAdmin):
    list_display = ['id', 'entidade', 'modulo', 'estado','created_at', 'updated_at', 'is_deleted']
    list_display_links = ('id', 'entidade', 'modulo',)
    search_fields = ['entidade', 'modulo',]
admin.site.register(EntidadeModulo, EntidadeModuloAdmin)

class TipoEntidadeModuloAdmin(admin.ModelAdmin):
    list_display = ['id', 'tipo_entidade', 'modulo', 'estado', 'created_at', 'updated_at', 'is_deleted']
    list_display_links = ('id', 'tipo_entidade', 'modulo',)
    search_fields = ['tipo_entidade', 'modulo',]
admin.site.register(TipoEntidadeModulo, TipoEntidadeModuloAdmin)


from django.urls import path
from rest_framework_simplejwt.views import (TokenRefreshView,)
from django.urls import path, include
from .views import *

from django.urls import path, include
from .import viewsApi, views
from rest_framework import routers

router = routers.DefaultRouter()
router.register(r'ficheiros', viewsApi.FicheiroAPIView)
router.register(r'idiomas', viewsApi.IdiomaAPIView)
router.register(r'traducaos', viewsApi.TraducaoAPIView)
router.register(r'inputs', viewsApi.InputAPIView)
router.register(r'strings', viewsApi.StringAPIView)
router.register(r'users', viewsApi.UsuarioAPIView)

router.register(r'tipoEntidades', viewsApi.TipoEntidadeAPIView)
router.register(r'entidades', viewsApi.EntidadeAPIView)
router.register(r'sucursals', viewsApi.SucursalAPIView)
router.register(r'grupos', viewsApi.GrupoAPIView)
router.register(r'permissions', viewsApi.PermissionAPIView)
router.register(r'modelos', viewsApi.ModeloAPIView)




   


urlpatterns = [
    path('', include(router.urls)),

    path("register/numero/i", getPhoneNumberRegistered.as_view(), name="OTP_Sem_Limite_De_Tempo"),
    path("register/numero/l", getPhoneNumberRegistered_TimeBased.as_view(), name="OTP_Com_Limite_De_Tempo"),

    path('register/', RegisterView.as_view(), name="register"),
    path('login/', LoginAPIView.as_view(), name="login"),
    path('logout/', LogoutAPIView.as_view(), name="logout"),
    path('me/', MeAPIView.as_view(), name="me"),
    path('change_password_email/', ChangePasswordEmailAPIView.as_view(), name="change_password_email"),
    path('email_verify/', VerifyEmail.as_view(), name="email_verify"),
    path('refresh_token/', TokenRefreshView.as_view(), name='refresh_token'),



    path('logins/', LoginsAPIView.as_view(), name="logins"),
    path('request_reset_email/', RequestPasswordResetEmail.as_view(), name="request_reset_email"),
    path('password_reset/<uidb64>/<token>/', PasswordTokenCheckAPI.as_view(), name='password_reset_confirm'),# ainda
    path('password_reset_complete', SetNewPasswordAPIView.as_view(), name='password_reset_complete'),
    path('email', Mail.as_view(), name='email'),
    path('change_password_mobile/', ChangePasswordEmailAPIView.as_view(), name="change_password_mobile"),

]









import re
import importlib
from django.apps import apps
from django.core.cache import cache
from ..models import Idioma, Traducao  # import apenas os modelos necessários

CACHE_TIMEOUT = 60 * 60  # 1 hora


class Translate:

    @staticmethod
    def tdc(request, texto):
        """
        Função equivalente à função JS 'traducao'.
        Procura a tradução direta e aplica replace_traducao se necessário.
        """

        # 1️⃣ Obtém traduções
        translations = Translate.getTranslations(request)
        if not translations:
            return texto

        # 2️⃣ Normaliza a chave
        chave = texto.lower().strip()

        # 3️⃣ Busca tradução direta
        traducao_direta = translations.get(chave)
        if not traducao_direta:
            return texto

        # 4️⃣ Aplica placeholders se existirem
        return Translate.replace_traducao(texto, traducao_direta)

    @staticmethod
    def getTranslations(request):
        """
        Obtém traduções do cache, banco de dados e módulos lang dos apps.
        """
        lang_id = request.headers.get('L')
        if not lang_id:
            return {}

        try:
            idioma = Idioma.objects.get(id=lang_id)
        except Idioma.DoesNotExist:
            return {}

        lang_code = idioma.code.lower().replace('-', '')
        cache_key = f"traducao:{lang_code}"

        # 1️⃣ Tenta obter do cache
        cached = cache.get(cache_key)
        if cached is not None:
            return cached

        traducoes = {}

        # 2️⃣ Base de dados
        db_traducoes = Traducao.objects.filter(idioma_id=idioma.id)
        for tra in db_traducoes:
            traducoes[tra.chave.lower().strip()] = tra.traducao  # normaliza a chave

        # 3️⃣ Módulos lang dos apps
        for app in apps.get_app_configs():
            module_name = f"{app.name}.lang.{lang_code}"
            try:
                file_traducoes = importlib.import_module(module_name)
            except ModuleNotFoundError:
                continue

            if hasattr(file_traducoes, "key_value") and isinstance(file_traducoes.key_value, dict):
                # normaliza as chaves dos módulos também
                traducoes.update({k.lower().strip(): v for k, v in file_traducoes.key_value.items()})

        # 4️⃣ Salva no cache
        cache.set(cache_key, traducoes, CACHE_TIMEOUT)

        return traducoes

    @staticmethod
    def captura(texto=''):
        """
        Captura placeholders do tipo '%- valor -%' no texto.
        """
        return re.findall(r'\s*%-\s*[\w\s-]+\s*-%\s*', texto)

    @staticmethod
    def replace_traducao(texto='', texto_de_traducao=''):
        """
        Substitui placeholders no texto_de_traducao pelos valores capturados em texto.
        """
        valores = Translate.captura(texto)
        if not valores:
            return texto_de_traducao

        i = 0

        def replacer(_):
            nonlocal i
            v = valores[i] if i < len(valores) else None
            i += 1
            return (' ' + v.replace('%-', '').replace('-%', '').strip() + ' ') if v else ''

        return re.sub(r'\s*%-\s*[\w\s-]+\s*-%\s*', replacer, texto_de_traducao)



class Translation:
    @staticmethod
    def encrypt(message, key):
        encrypted = ""
        for char in message:
            if char.isalpha():
                shifted = ord(char) + key
                if char.isupper():
                    if shifted > ord('Z'):
                        shifted -= 26
                    elif shifted < ord('A'):
                        shifted += 26
                else:
                    if shifted > ord('z'):
                        shifted -= 26
                    elif shifted < ord('a'):
                        shifted += 26
                encrypted += chr(shifted)
            else:
                encrypted += char
        return encrypted

    @staticmethod
    def decrypt( message, key):
        return Translation.encrypt(message, -key)





from django.contrib.auth.models import Permission, Group
from rest_framework import permissions
from rest_framework import status
from django.conf import settings
from rest_framework.response import Response
from .classes.Translate import Translate
import inspect
from .models import *
from functools import wraps



def check_permission(role, request):
    role = role or []

    tipo_entidade_id = request.headers.get('ET')
    entidade_id = request.headers.get('E')
    sucursal_id = request.headers.get('S')
    grupo_id = request.headers.get('G')

    if not Entidade.objects.filter( id=entidade_id, tipo_entidade__id=tipo_entidade_id ).exists():
        return False

    if not EntidadeUser.objects.filter( user=request.user, entidade_id=entidade_id ).exists():
        return False

    if not Sucursal.objects.filter( id=sucursal_id, entidade__tipo_entidade__groups__id=grupo_id ).exists():
        return False

    if not SucursalUser.objects.filter( user=request.user, sucursal_id=sucursal_id ).exists():
        return False

    if not SucursalUserGroup.objects.filter( user=request.user, sucursal_id=sucursal_id, group_id=grupo_id ).exists():
        return False

    try:
        grupo = Group.objects.get(id=grupo_id)
    except Group.DoesNotExist:
        return False

    return grupo.permissions.filter(codename__in=role).exists()



def hasPermission(role=None):
    def decorator(view_func):
        @wraps(view_func)
        def wrapper(self, request, *args, **kwargs):
            if check_permission(role, request):
                return view_func(self, request, *args, **kwargs)

            txt = Translate.tdc( request.query_params.get('lang'), 'Permission denied')
            return Response( {'alert_error': txt}, status=status.HTTP_403_FORBIDDEN )
        return wrapper
    return decorator

def isPermited(role=None, request=None):
    return check_permission(role, request)



import string
import random
from .Translation import Translation

from django.conf import settings

class FullPath:
    @staticmethod
    def generate_file_url_token():
        characters = string.ascii_letters + string.digits
        code = ''
        translacao = random.choice(string.digits.replace('0', ''))
        encode_key = int(random.choice(string.digits.replace('0', '')))
        code = code.join(translacao)
        chave_encriptada = Translation.encrypt(settings.URL_FILE_KEY, encode_key)

        for i in range(len(chave_encriptada)):
            separacao = ''
            for j in range(int(translacao)):
                separacao = separacao + random.choice(characters)
            code = code + separacao + chave_encriptada[i]

        code = code + str(encode_key)
        return code

    @staticmethod
    def get_key_file_url_token(code):
        try:
            translacao = int(code[0])
        except Exception:
            return ''
        try:
            encode_key = int(code[-1])
        except Exception:
            return ''
            
        chave = ''
        j = 0
        tamanho = (len(code[1:-1]) / translacao)
        for i in range(int(tamanho)):
            j = j + 1
            help = (translacao * j) + j
            try:
                chave = chave + code[1:-1][help - 1]
            except Exception as e:
                pass
        return Translation.decrypt(chave, encode_key)

    @staticmethod
    def url(request, name):
        gerado = FullPath.generate_file_url_token()
        return request.build_absolute_uri(name) +'?token=' + gerado 

    



from rest_framework import serializers
from .classes.FullPath import FullPath
from .models import *
from drf_writable_nested import WritableNestedModelSerializer
from django.contrib.auth.models import Group, Permission
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.contenttypes.models import ContentType

from rest_framework_simplejwt.exceptions import TokenError
from rest_framework_simplejwt.tokens import RefreshToken

from rest_framework.exceptions import AuthenticationFailed
from django.core.validators import validate_email

from django.contrib.auth.backends import BaseBackend
from django.contrib.auth import get_user_model
from django.db.models import Q





class TraducaoSerializer(serializers.ModelSerializer):
    class Meta:
        model = Traducao
        fields = ['id', 'chave', 'traducao', 'idioma']


def authenticate(value=None, password=None):
    User = get_user_model()
    try:
        user = User.objects.get(
            Q(mobile=value) | Q(email=value) | Q(username=value)
        )
    except User.DoesNotExist:
        return None
    if user.check_password(password):
        return user
   
    return None

class InputSerializer(serializers.ModelSerializer):
    class Meta:
        model = Input
        fields = ['id', 'nome']
        
class StringSerializer(serializers.ModelSerializer):
    class Meta:
        model = String
        fields = ['id', 'texto']

class UserLoginSerializer( serializers.ModelSerializer):
    class Meta:
        model = UserLogin
        fields = ['id', 'user', 'info', 'dispositivo', 'local_lat','local_lon', 'local_nome','data', 'hora', 'is_blocked']


class RegisterSerializer(serializers.ModelSerializer):

    default_error_messages = {
        'username': 'The username should only contain alphanumeric characters'}

    class Meta:
        model = User
        fields = ('id', 'email', 'username', 'password', 'mobile')
        

    def validate(self, attrs):
        email = attrs.get('email', '')
        username = attrs.get('username', '')

        if not username.isalnum():
            raise serializers.ValidationError(
                self.default_error_messages)
        return attrs

    def create(self, validated_data):
        return User.objects.create_user(**validated_data)


class EmailVerificationSerializer(serializers.ModelSerializer):
    token = serializers.CharField(max_length=555)
    class Meta:
        model = User
        fields = ['token']

class MeSerializer(serializers.ModelSerializer):
    perfil = serializers.SerializerMethodField()
    def get_perfil(self, obj):
        request = self.context.get('request')
        if request is None:
            return None

        return {
            'name': str(obj.perfil.url).split('/')[-1], 
            'url': FullPath.url(request, obj.perfil.url), 
            'ext': str(obj.perfil.url).split('.')[-1]
        }
    class Meta:
        model = User
        fields = ['id', 'email', 'username', 'perfil', 'mobile']


class LoginSerializer(serializers.ModelSerializer):
    email = serializers.EmailField(max_length=255, min_length=3)

    password = serializers.CharField(
        max_length=68, min_length=6, write_only=True)
    username = serializers.CharField(
        max_length=255, min_length=3, read_only=True)


    tokens = serializers.SerializerMethodField()

    class Meta:
        model = User
        fields = ['id', 'email', 'password', 'username', 'mobile','tokens']

    def get_tokens(self, obj):
        user = User.objects.get(id=obj['id'])
        return {
            'refresh': user.tokens()['refresh'],
            'access': user.tokens()['access']
        }
    


    def validate(self, attrs):
        email = attrs.get('email', '')
        password = attrs.get('password', '')
        email = email.replace('+', '')
        email = email.replace("@paravalidar.com", '')
        email = email.strip()
       
        if email.isnumeric():
            user = authenticate(value=email, password=password)
        else:
            user =  authenticate(value=email, password=password)
       

        if not user:
            raise AuthenticationFailed('Invalid credentials, try again')
        if not user.is_active:
            raise AuthenticationFailed('Account disabled, contact admin')
        if not user.is_verified:
            raise AuthenticationFailed('Email is not verified')

        return {
            'id': user.id,
            'email': user.email,
            'username': user.username,
            'mobile': user.mobile,
            'tokens': user.tokens
        }

        return super().validate(attrs)


class ResetPasswordEmailRequestSerializer(serializers.Serializer):
    email = serializers.EmailField(min_length=2)

    redirect_url = serializers.CharField(max_length=500, required=False)

    class Meta:
        fields = ['email']


class SetNewPasswordSerializer(serializers.Serializer):
    password = serializers.CharField(
        min_length=6, max_length=68, write_only=True)
    token = serializers.CharField(
        min_length=1, write_only=True)
    uidb64 = serializers.CharField(
        min_length=1, write_only=True)

    class Meta:
        fields = ['password', 'token', 'uidb64']

    def validate(self, attrs):
        try:
            password = attrs.get('password')
            token = attrs.get('token')
            uidb64 = attrs.get('uidb64')

            id = force_str(urlsafe_base64_decode(uidb64))
            user = User.objects.get(id=id)
            if not PasswordResetTokenGenerator().check_token(user, token):
                raise AuthenticationFailed('O link de redefinição é inválido', 401)

            user.set_password(password)
            user.save()

            return (user)
        except Exception as e:
            raise AuthenticationFailed('O link de redefinição é inválido', 401)
        return super().validate(attrs)


class LogoutSerializer(serializers.Serializer):
    refresh = serializers.CharField()

    default_error_message = {
        'bad_token': ('Token is expired or invalid')
    }

    def validate(self, attrs):
        self.token = attrs['refresh']
        return attrs

    def save(self, **kwargs):
        try:
            RefreshToken(self.token).blacklist()
        except TokenError:
            self.fail('bad_token') 
        pass


class EntidadeSerializer(serializers.ModelSerializer):
    logo = serializers.SerializerMethodField()
    def get_logo(self, object):
        request = self.context.get('request')
        return {'name': str(object.logo).split('/')[-1], 'url': FullPath.url(request, object.logo.name), 'ext': str(object.logo).split('.')[-1]}

    class Meta:
        model = Entidade
        fields = ['id', 'nome', 'logo', 'ipu', 'groups', 'admins', 'modelos', 'display_logo', 'display_qr', 'display_bar',  'tipo_entidade', 'entidade_bancaria',  'disc_space', 'disc_used_space', 'disc_free_space', 'rodape', 'estado', 'created_at', 'updated_at', 'is_deleted']


class EntidadeUserSerializer(serializers.ModelSerializer):
    user = serializers.SerializerMethodField()
    @classmethod
    def get_user(self, object):
        """getter method to add field get_user"""
        pessoa = Pessoa.objects.filter(user__id=object.user.id).first()
        user = User.objects.filter(id=object.user.id).first()
        pessoa = PessoaSerializer(pessoa)
        user = UserSerializer(user)
        return {'pessoa': pessoa.data, 'user': user.data}

    class Meta:
        model = EntidadeUser
        fields = ['id', 'entidade', 'user']


class EntidadeLogoSerializer(serializers.ModelSerializer):
    class Meta:
        model = Entidade
        fields = ['logo']


class SucursalSerializer(serializers.ModelSerializer):
    url = serializers.SerializerMethodField()
    def get_url( self, object):
        suc = []
        sucursalUserGroups = SucursalGroup.objects.all().filter(sucursal__id=object.id)

        sucursal = Sucursal.objects.get(id=object.id)
        entidade = Entidade.objects.get(id=sucursal.entidade.id)

        return str(entidade.tipo_entidade.id)+'/'+str(sucursal.entidade.id)+'/'+str(sucursal.id)+'/'

    class Meta:
        model = Sucursal
        fields = ['id', 'nome', 'entidade', 'endereco', 'l_latitude', 'l_longetude', 'url']


class IdiomaSerializer(serializers.ModelSerializer):
    class Meta:
        model = Idioma
        fields = ['id', 'nome', 'code', 'estado']





class PermissionSerializer(serializers.ModelSerializer):
    content_type_model = serializers.CharField()
    content_type_app = serializers.CharField()
    class Meta:
        model = Permission
        fields = ['id', 'name', 'codename', 'content_type', 'content_type_model', 'content_type_app']


class GrupoSerializer(serializers.ModelSerializer):
    class Meta:
        model = Group
        fields = ['id', 'name']  # , 'permissions'


class ContentTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = ContentType
        fields = ['id', 'app_label', 'model']

class TipoEntidadeSerializer(serializers.ModelSerializer):
    icon = serializers.SerializerMethodField()
    def get_icon( self, object):
        request = self.context.get('request')
        return {'name': str(object.icon).split('/')[-1], 'url': FullPath.url(request, object.icon.name),
                'ext': str(object.icon).split('.')[-1]}

    groups = serializers.SerializerMethodField()
    def get_groups( self, object):
        request = self.context.get('request')
        groups = GrupoSerializer(object.groups.all(), many=True)
        return groups.data

    class Meta:
        model = TipoEntidade
        fields = ['id', 'nome', 'estado', 'icon','label', 'groups',  'crair_entidade', 'link', 'header', 'menuEsquerdo', 'footer', 'created_at',
                  'created_at_time', 'updated_at', 'updated_at_time', 'is_deleted']
        read_only_fields = ['icon']

class UserSerializer(serializers.ModelSerializer):
    perfil = serializers.SerializerMethodField()
    def get_perfil(self, obj):
        request = self.context.get('request')
        print(obj.perfil)

        if request is None:
            return None
        return {
            'name': str(obj.perfil.url).split('/')[-1], 
            'url': FullPath.url(request, obj.perfil.url), 
            'ext': str(obj.perfil.url).split('.')[-1]
        }
    class Meta:
        model = User
        fields = ['id', 'perfil', 'username', 'email', 'groups', 'is_verified', 'is_active']


class FicheiroSerializer(serializers.ModelSerializer):
    ficheiro = serializers.SerializerMethodField(source='get_ficheiro_value')
    size = serializers.SerializerMethodField(source='get_size__value')

    def get_ficheiro(self, object):
        request_ = None
        return {'name': str(object.ficheiro).split('/')[-1], 'url': FullPath.url(request_, object.ficheiro.name), 'ext': str(object.ficheiro).split('.')[-1], 'size':object.ficheiro.size}

    def get_size(self, object):
        request_ = None
        return object.ficheiro.size

    class Meta:
        model = Ficheiro
        fields =['id', 'ficheiro', 'size', 'modelo', 'estado', 'chamador', 'funcionalidade', 'sucursal', 'entidade', 'created_at', 'updated_at', 'created_at_time', 'updated_at_time', 'is_deleted']


class FicheiroGravarSerializer(serializers.ModelSerializer):
    class Meta:
        model = Ficheiro
        fields =['id', 'ficheiro', 'size', 'modelo', 'estado', 'chamador', 'funcionalidade', 'sucursal', 'entidade', 'created_at', 'updated_at', 'created_at_time', 'updated_at_time', 'is_deleted']




import json

from django.http import HttpResponse
from rest_framework.response import Response
from rest_framework import status

from .classes.FullPath import FullPath
from .classes.Translate import Translate
from .models import FrontEnd
from django.conf import settings


class CoreMidlleware:
    def __init__(self, get_response):
        self.get_response = get_response
    def __call__(self, request):
        template = str(request.META['PATH_INFO']).split('/')[1]
        if not (template in settings.AUTH_URL):
            if str(request.headers.get('FEK')) != 'None' and str(request.headers.get('FEP')) != 'None':
                if not FrontEnd.objects.filter(fek=str(request.headers.get('FEK')), fep=str(request.headers.get('FEP'))).first():
                    txt = "code 10001 " + Translate.tdc(request.META['QUERY_STRING'], 'Nao autorizado Bad Credentials') + " " +  request.META['REMOTE_ADDR'] +" "+str(request.headers.get('FEK')) +" "+str(request.headers.get('FEP'))
                    return HttpResponse(txt, status=status.HTTP_401_UNAUTHORIZED, content_type='application/json')
            else:
                if request.META['REMOTE_ADDR'] not in settings.ALLOWED_HOSTS:
                    txt = "code 10002 " + Translate.tdc(request.META['QUERY_STRING'], 'Nao autorizado No Credentials') + " " + request.META['REMOTE_ADDR']
                    return HttpResponse(txt, status=status.HTTP_401_UNAUTHORIZED, content_type='application/json')

        response = self.get_response(request)
        return response

class CoreFileMidlleware:
    def __init__(self, get_response):
        self.get_response = get_response
    def __call__(self, request):

        txt = Translate.tdc(request.META['QUERY_STRING'], 'Nao autorizado')
        qs = request.META['QUERY_STRING']
        path = request.META['PATH_INFO'].split('/')[1]
        token = None

        if path == settings.MEDIA_URL.split('/')[1]:
            if str(qs).__contains__('='):
                if str(qs).__contains__('&'):
                    lang = qs.split('&')
                    for l in lang:
                        if l.split('=')[0] == 'token':
                            token = l.split('=')[1]
                else:
                    if qs.split('=')[0] == 'token':
                        token = qs.split('=')[1]
            if token:
                if FullPath.get_key_file_url_token(token) == settings.URL_FILE_KEY:
                    response = self.get_response(request)
                    return response
            return HttpResponse(txt, status=status.HTTP_401_UNAUTHORIZED, content_type='application/json')
        response = self.get_response(request)
        return response



from rest_framework import viewsets
from rest_framework import filters
from .serializers import *
from .models import *
from rest_framework.decorators import action
from django.db.models import F
from rest_framework.response import Response
from rest_framework import status
import importlib.util
from django.apps import apps
from .decorators import hasPermission, isPermited
from django.core.cache import cache
import importlib



class  TraducaoAPIView(viewsets.ModelViewSet):
    search_fields = ['idioma__nome','chave']
    filter_backends = (filters.SearchFilter,)
    serializer_class = IdiomaSerializer
    queryset = Idioma.objects.all()

    def get_queryset(self):
        return self.queryset.filter().order_by('chave')


CACHE_TIMEOUT = 60 * 60  # 1 hora

@action(detail=True, methods=['GET'])
def traducoes(self, request, *args, **kwargs):
    transformer = self.get_object()
    lang_code = str(transformer.code).lower().replace('-', '')
    cache_key = f"traducao:{lang_code}"

    # 1️⃣ Tenta obter do cache
    cached = cache.get(cache_key)
    if cached is not None:
        return Response(cached, status=status.HTTP_200_OK)

    # 2️⃣ Caso não exista cache, constrói as traduções
    traducoes = {}

    # Base de dados
    db_traducoes = Traducao.objects.filter(idioma_id=transformer.id)
    for tra in db_traducoes:
        traducoes[tra.chave] = tra.traducao

    # Módulos lang
    for app in apps.get_app_configs():
        module_name = f"{app.name}.lang.{lang_code}"

        try:
            modulo = importlib.import_module(module_name)
        except ModuleNotFoundError:
            continue

        if hasattr(modulo, "key_value"):
            traducoes.update(modulo.key_value)

    # 3️⃣ Guarda no cache
    cache.set(cache_key, traducoes, CACHE_TIMEOUT)

    return Response(traducoes, status=status.HTTP_200_OK)


    @action(
        detail=True,
        methods=['GET'],
    )
    def getTraducao(self, request, id):
        traducaos = Traducao.objects.filter(idioma=id)
        idioma = Idioma.objects.get(id=id)

        traducao_ = []
        for traducao in traducaos:
            traducao_.append({traducao.chave: traducao.traducao})

        if idioma.code:
            pass
        else:
            idioma.code = "PT-PT"
        # iterate over each line as a ordered dictionary and print only few column by column name
        try:
            with open(str(os.getcwd()) + '/core/lang/{}.csv'.format(idioma.code), 'r') as read_obj:
                csv_dict_reader = DictReader(read_obj)
                for row in csv_dict_reader:
                    traducao_.append({row['chave']: row['traducao']})

        except :
            pass
        return Response(traducao_,  status.HTTP_200_OK)


class  FicheiroAPIView(viewsets.ModelViewSet):

    search_fields = ['id','ficheiro']
    filter_backends = (filters.SearchFilter,)
    serializer_class = FicheiroSerializer
    queryset = Ficheiro.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        return self.queryset.filter().order_by('-id')
    # A implementaçao desse codigo faz com que nao se returne o full path de im ficheiro
    def retrieve(self, request, id, *args, **kwargs):
        try:
            transformer = self.get_object()
            entidade = FicheiroSerializer(transformer)
            return Response(entidade.data, status=status.HTTP_200_OK)
        except Http404:
            pass
        return Response( status=status.HTTP_404_NOT_FOUND)


    def destroy(self, request, id, *args, **kwargs):
        try:
            instance = self.get_object()
            DiscManegar.recoverSpace(instance.entidade.id, instance)
            self.perform_destroy(instance)
        except Http404:
            pass
        return Response(status=status.HTTP_204_NO_CONTENT)

    
    def list(self, request, *args, **kwargs):
        self._paginator = None
        queryset = self.filter_queryset(self.get_queryset().filter().order_by('-id'))
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)


    def update(self, request,id,  *args, **kwargs):
        transformer = self.get_object()
        entidade = FicheiroSerializer(transformer, data=request.data)
        if entidade.is_valid(raise_exception=True):
            entidade.save()
            return Response(entidade.data, status=status.HTTP_201_CREATED)
        else:
            return Response(entidade.errors, status=status.HTTP_400_BAD_REQUEST)

    def create(self, request, *args, **kwargs):
        tipo_entidade_id = request.headers.get('ET')
        entidade_id = request.headers.get('E')
        sucursal_id = request.headers.get('S')
        grupo_id = request.headers.get('G')
        request.data['entidade'] = entidade_id
        request.data['sucursal'] = sucursal_id
        uploaded_file = request.FILES['ficheiro']
        
        if entidade_id:
            DiscManegar.freeSpace(entidade_id, request.FILES['ficheiro'])

        request.data['size'] = uploaded_file.size

        ficheiro = FicheiroGravarSerializer(data=request.data)
        if ficheiro.is_valid(raise_exception=True):
            ficheiro.save()
            ficheiro = FicheiroSerializer(Ficheiros.objects.get(id=ficheiro.data['id']))
            return Response(ficheiro.data, status=status.HTTP_201_CREATED)
        else:
            return Response(ficheiro.errors, status=status.HTTP_400_BAD_REQUEST)


    @action(
        detail=True,
        methods=['GET'],
    )
    def sucursals(self, request, id):
        sucursals = FicheiroSerializer.objects.filter(entidade__id=id)
        suc = []
        for sucursal in sucursals:
            try:
                suc.append({'id': sucursal.id, 'nome': sucursal.nome})
            except:
                pass
        return Response(suc)



class ModeloAPIView(viewsets.ModelViewSet):
    search_fields = ['id']
    filter_backends = (filters.SearchFilter,)
    serializer_class = ContentTypeSerializer
    queryset = ContentType.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        return self.queryset.filter().order_by('app_label','model')
    
    def list(self, request, *args, **kwargs):
        self._paginator = None
        queryset = self.filter_queryset(self.get_queryset().filter().order_by('app_label', 'model'))
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)


class InputAPIView(viewsets.ModelViewSet):
    search_fields = ['id','nome']
    filter_backends = (filters.SearchFilter,)
    serializer_class = InputSerializer
    queryset = Input.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        self._paginator = None
        return self.queryset.filter().order_by('nome')
    
    @action(
        detail=True,
        methods=['GET'],
    )
    def strings(self, request, id, *args, **kwargs):
        r = []
        try:
            st = InputString.objects.filter(input__id=id)
            for i in st:
                string = StringSerializer(String.objects.get(id=i.string.id))
                r.append(string.data)
            return Response(r, status.HTTP_200_OK)
        except:
            return Response(r, status.HTTP_200_OK)
        
    @action(
        detail=True,
        methods=['POST'],
    )
    def addString(self, request, id):
        
        string = String.objects.create(texto=request.data['texto'])
        input = Input.objects.get(id=id)
        InputString.objects.create(string = string , input = input)
   
        per= {'id': string.id, 'nome': string.texto, 'nomeseparado':input.nome, 'alert_info': 'Permicao <b>' +string.texto + '</b> foi removido consucesso'}
        return Response(per,status.HTTP_201_CREATED )
    
    


class StringAPIView(viewsets.ModelViewSet):
    search_fields = ['id','texto']
    filter_backends = (filters.SearchFilter,)
    serializer_class = StringSerializer
    queryset = String.objects.all()
    lookup_field = "id"


    def get_queryset(self):
        self._paginator = None
        if (self.request.query_params.get('input')):
            r = []
            try:
                st = InputString.objects.filter(input__nome=self.request.query_params.get('input'))
                for i in st:
                    string = StringSerializer(String.objects.get(id=i.string.id))
                    r.append(string.data)
                return r
            except:
                return r
        else:
            return self.queryset.filter().order_by('texto')
    
    @action(
        detail=True,
        methods=['GET'],
    )
    def inputs(self, request, id, *args, **kwargs):
        r = []
        try:
            st = InputString.objects.filter(string__id=id)
            for i in st:
                input = InputSerializer(Input.objects.get(id=i.input.id))
                r.append(input.data)
            return Response(r, status.HTTP_200_OK)
        except:
            return Response(r, status.HTTP_200_OK)
        
    @action(
        detail=True,
        methods=['POST'],
    )
    def addToInput(self, request, id):

        string = String.objects.get(id=id)
        input = Input.objects.get(id=request.data['id'])
        inpStr = InputString()
        inpStr.input = input
        inpStr.string = string
        inpStr.save()
    
        per= {'id': string.id, 'nome': string.texto, 'nomeseparado':input.nome, 'alert_info': 'Permicao <b>' +string.texto +  '</b> foi Adicionado consucesso'}
        return Response(per,status.HTTP_201_CREATED )


    @action(
        detail=True,
        methods=['POST'],
    )
    def removeFromInput(self, request, id):
        print(id,request.data['id'])
        InputString.objects.get(string__id = id , input__id = request.data['id']).delete()
        print(inpStr)
        string = String.objects.get(id=inpStr.string.id)
        input = Input.objects.get(id=inpStr.input.id)
    
        per= {'id': string.id, 'nome': string.texto, 'nomeseparado':input.nome, 'alert_info': 'Permicao <b>' +string.texto + '</b> foi removido consucesso'}
        return Response(per,status.HTTP_201_CREATED )


class  IdiomaAPIView(viewsets.ModelViewSet):
    paginator = None
    search_fields = ['id','nome']
    filter_backends = (filters.SearchFilter,)
    serializer_class = IdiomaSerializer
    queryset = Idioma.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        return self.queryset.filter().order_by('nome')


    def retrieve(self, request, id, *args, **kwargs):
        try:
            transformer = self.get_object()
            idioma = IdiomaSerializer(transformer)
            return Response(idioma.data, status=status.HTTP_200_OK)
        except Http404:
            pass
        return Response( status=status.HTTP_404_NOT_FOUND)


    def destroy(self, request, id, *args, **kwargs):
        try:
            instance = self.get_object()
            self.perform_destroy(instance)
        except Http404:
            pass
        return Response(status=status.HTTP_204_NO_CONTENT)


    def update(self, request,id,  *args, **kwargs):
        transformer = self.get_object()

        idioma = IdiomaSerializer(transformer, data=request.data)
        if idioma.is_valid(raise_exception=True):
            idioma.save()
            add={'alert_success': '%-'+ request.data['nome'] +'-% foi actualizado com sucesso'}
            data = json.loads(json.dumps(idioma.data))
            data.update(add)

            return Response(data, status=status.HTTP_202_ACCEPTED)
        else:
            return Response(idioma.errors, status=status.HTTP_400_BAD_REQUEST)

    def create(self, request, *args, **kwargs):
        request.data['admin'] = request.user.id
        idioma = IdiomaSerializer(data=request.data)
        if idioma.is_valid(raise_exception=True):
            idioma.save()
            add = {'alert_success': '%-'+ request.data['nome'] +'-% foi criado com sucesso'}
            data = json.loads(json.dumps(idioma.data))
            data.update(add)

            return Response(data, status=status.HTTP_202_ACCEPTED)
        else:
            return Response(idioma.errors, status=status.HTTP_400_BAD_REQUEST)


class UsuarioAPIView(viewsets.ModelViewSet):
    search_fields = ['id','username']
    filter_backends = (filters.SearchFilter,)
    serializer_class = UserSerializer
    queryset = User.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        if (self.request.query_params.get('allPaginado')):
            return self.queryset.filter().order_by('id')
        else:
            self._paginator = None
            return self.queryset.filter().order_by('id')

    @action(
        detail=True,
        methods=['GET'],
    )
    def userEntidades(self, request, id, *args, **kwargs):
        user = User.objects.get(id=id)
        user = UserSerializer(user)
        tipo_entidade_id = request.headers.get('ET')
        entidade_id = request.headers.get('E')
        sucursal_id = request.headers.get('S')
        grupo_id = request.headers.get('G')

        ar = []
        userEntidades = EntidadeUser.objects.filter(user__id=id, entidade__tipo_entidade__id=tipo_entidade_id)
        if (userEntidades):
            for userEntidade in userEntidades:
                entidade = Entidade.objects.get(id=userEntidade.entidade.id)
                entidade = EntidadeSerializer(entidade, context={'request': request})
                ar.append({'id': entidade.data['id'], 'tipoEntidade': entidade.data['tipo_entidade'],  'nome': entidade.data['nome'], 'created_at': entidade.data['created_at'].split('-')[0], 'logo': entidade.data['logo']['url']})
           

        return Response(ar, status.HTTP_200_OK)

    @action(
        detail=True,
        methods=['GET'],
    )
    def logins(self, request, id, *args, **kwargs):
        userLogin = UserLogin.objects.filter(user_id = id).order_by('-data', 'hora')
        userLogins = UserLoginSerializer(userLogin, many=True)
        return Response(userLogins.data, status=status.HTTP_200_OK)
    
    
    @action(
        detail=True,
        methods=['GET'],
    )
    def userSucursals(self, request, id, *args, **kwargs):
        user = User.objects.get(id=id)
        user = UserSerializer(user)
        tipo_entidade_id = request.headers.get('ET')
        entidade_id = request.headers.get('E')
        sucursal_id = request.headers.get('S')
        grupo_id = request.headers.get('G')

        ar = []
        userSucursals = SucursalUser.objects.filter(user__id=id, sucursal__entidade__tipo_entidade__id=tipo_entidade_id, sucursal__entidade__id=entidade_id)
        if (userSucursals):
            for userSucursal in userSucursals:
                sucursal = Sucursal.objects.get(id=userSucursal.sucursal.id)
                sucursal = SucursalSerializer(sucursal)
                ar.append({'id': sucursal.data['id'], 'nome': sucursal.data['nome']})

        return Response(ar, status.HTTP_200_OK)
    
    @action(
        detail=True,
        methods=['POST'],
    )
    def addUserSucursal(self, request, id, *args, **kwargs):
        user = User.objects.get(id=id)
        tipo_entidade_id = request.headers.get('ET')
        entidade_id = request.headers.get('E')
        sucursal_id = request.headers.get('S')
        grupo_id = request.headers.get('G')
        sucursal = Sucursal.objects.get(id= request.data['sucursal'])

        ar = []
        userSucursals = SucursalUser.objects.filter(user__id=id, sucursal__id= sucursal.id,  sucursal__entidade__tipo_entidade__id=tipo_entidade_id, sucursal__entidade__id=entidade_id)
        if (len(userSucursals) <= 1):
            su = SucursalUser()
            su.user = user
            su.sucursal  = sucursal
            su.save()
            # data = json.loads(json.dumps(paciente.data, cls=DjangoJSONEncoder))
        add = {'alert_success':  '<b>' + sucursal.nome+ '</b> foi adicionado com sucesso'}
            # data.update(add)
        return Response(add, status = status.HTTP_201_CREATED)
    
    @action(
        detail=True,
        methods=['POST'],
    )
    def removeUserSucursal(self, request, id, *args, **kwargs):
        user = User.objects.get(id=id)
        tipo_entidade_id = request.headers.get('ET')
        entidade_id = request.headers.get('E')
        sucursal_id = request.headers.get('S')
        grupo_id = request.headers.get('G')
        sucursal = Sucursal.objects.get(id= request.data['sucursal'])

        userSucursals = SucursalUser.objects.get(user__id=id, sucursal__id= sucursal.id, sucursal__entidade__tipo_entidade__id=tipo_entidade_id, sucursal__entidade__id=entidade_id)
        userSucursals.delete()
        add = {'alert_success': '<b>' + sucursal.nome+ '</b> foi removido com sucesso'}
        return Response(add, status = status.HTTP_200_OK)

    @action(
        detail=True,
        methods=['GET'],
    )
    def userGrupos(self, request, id, *args, **kwargs):
        user = User.objects.get(id=id)
        user = UserSerializer(user)

        if self.request.query_params.get('sucursal') == 'nulo' or self.request.query_params.get('sucursal') == None:
            tipo_entidade_id = request.headers.get('ET')
            entidade_id = request.headers.get('E')
            sucursal_id = request.headers.get('S')
            grupo_id = request.headers.get('G')
        else:
            sucursal_id = self.request.query_params.get('sucursal')

        sucursalUserGroups = SucursalUserGroup.objects.filter(user__id=id, sucursal__id=sucursal_id)
        ar = []
        if (sucursalUserGroups):
            for sucursalUserGroup in sucursalUserGroups:
                group = Group.objects.get(id=sucursalUserGroup.group.id)
                ar.append({'id': group.id, 'name': group.name})

        if True:
            return Response(ar, status.HTTP_200_OK)
        return Response([], status.HTTP_400_BAD_REQUEST)

    @action(
        detail=True,
        methods=['GET'],
    )
    def userPermicoes(self, request, id, *args, **kwargs):
        user = User.objects.get(id=id)
        user = UserSerializer(user)
        tipo_entidade_id = request.headers.get('ET')
        entidade_id = request.headers.get('E')
        sucursal_id = request.headers.get('S')
        grupo_id = request.headers.get('G')

        group_id = grupo_id
        sucursalUserGroup = SucursalUserGroup.objects.filter(user__id = id, sucursal__id = sucursal_id, group__id = group_id)
        
        per = []
        if (sucursalUserGroup):
            grupo = Group.objects.get(id=sucursalUserGroup[0].group.id)
            permissions = grupo.permissions.all()

            for permission in permissions:
                per.append({'id': permission.id, 'nome': permission.codename, 'nomeseparado': permission.name})

        if True:
            return Response(per, status.HTTP_200_OK)
        return Response([], status.HTTP_400_BAD_REQUEST)


    @action(
        detail=True,
        methods=['GET'],
    )
    def userPessoa(self, request, id, *args, **kwargs):

        pessoa = Pessoa.objects.get(user__id=id)
        pessoa = PessoaSerializer(pessoa)
        if pessoa:
            return Response(pessoa.data, status.HTTP_200_OK)
        return Response([], status.HTTP_400_BAD_REQUEST)


    @action(
        detail=True,
        methods=['POST'],
    )
    def removerPerfil(self, request, id ):
        user = User.objects.get(id=id)
        group_id = request.data['perfil']['id']
        sucursal_id = request.data['sucursal_id']
        sucursalUserGroups = SucursalUserGroup.objects.filter(user__id=id, sucursal__id=sucursal_id, group__id= group_id).first()
        sucursalUserGroups.delete()
        ar = []
        sucursalUserGroups = SucursalUserGroup.objects.filter(user__id=id, sucursal__id=sucursal_id)
        if (sucursalUserGroups):
            for sucursalUserGroup in sucursalUserGroups:
                group = Group.objects.get(id=sucursalUserGroup.group.id)
                ar.append({'id': group.id, 'name': group.name})
                # print(ar)

        if True:
            return Response(ar, status.HTTP_200_OK)
        return Response([], status.HTTP_400_BAD_REQUEST)


    @action(
        detail=True,
        methods=['POST'],
    )
    def adicionarPerfil(self, request, id):
        user = User.objects.get(id=id)
        group_id = request.data['perfil']['id']
        group = Group.objects.get(id = group_id)
        sucursal_id = request.data['sucursal_id']
        sucursal = Sucursal.objects.get(id=sucursal_id)
        sucursalUserGroups = SucursalUserGroup.objects.filter(user__id=id, sucursal__id=sucursal_id, group__id= group_id).first()

        if None==sucursalUserGroups:
            sucursalUserGroup = SucursalUserGroup()
            sucursalUserGroup.user = user
            sucursalUserGroup.group = group
            sucursalUserGroup.sucursal = sucursal
            sucursalUserGroup.save()
        sucursalUserGroups = SucursalUserGroup.objects.filter(user__id=id, sucursal__id=sucursal_id)

        ar = []
        if (sucursalUserGroups):
            for sucursalUserGroup in sucursalUserGroups:
                group = Group.objects.get(id=sucursalUserGroup.group.id)
                ar.append({'id': group.id, 'name': group.name})
                # print(ar)

        if True:
            return Response(ar, status.HTTP_200_OK)
        return Response([], status.HTTP_400_BAD_REQUEST)




class TipoEntidadeAPIView(viewsets.ModelViewSet):
    search_fields = ['id','nome']
    filter_backends = (filters.SearchFilter,)
    
    serializer_class =TipoEntidadeSerializer
    queryset =TipoEntidade.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        if (self.request.query_params.get('all')):
            return self.queryset.filter().order_by('ordem')
        else:
            self._paginator = None
            return self.queryset.filter(estado='Activo').order_by('ordem')
    
    

    @action(
        detail=True,
        methods=['GET'],
    )
    def user_entidades(self, request, id):
        entidades = Entidade.objects.filter(tipo_entidade__id=id)
        ent = []
        for entidade in entidades:

            try:
                entidadeUser = EntidadeUser.objects.get( entidade=entidade, user=request.user)
                if entidadeUser:
                    logo = FullPath.url(request, entidade.logo.name)
                    ent.append({'id': entidade.id, 'nome': entidade.nome, 'logo': logo})
            except:
                pass
        return Response(ent , status=status.HTTP_200_OK)


    @action(
        detail=True,
        methods=['GET'],
    )
    def entidades(self, request, id):
        entidades = Entidade.objects.filter(tipo_entidade__id = id)

        ent = []
        for entidade in entidades:
            try:
                logo = str(entidade)
                ent.append({'id': entidade.id, 'nome': entidade.nome})
            except:
                pass
        return Response(ent , status=status.HTTP_200_OK)
    

    @action(
        detail=True,
        methods=['GET'],
    )
    def downloadPermission(self, request, id):
        te = TipoEntidade.objects.get(id=id)
        groups = db.reference('Premissions').child(te.nome).get()
        for key, permissions in groups.items():
            try:
                g = Group.objects.get(name=key)
                te.groups.add(g)
            except:
                g = Group.objects.create(name=key)
                te.groups.add(g)

        for group in te.groups.all():
            datas = db.reference('Premissions').child(te.nome).child(group.name).get()
            gr = Group.objects.get(id=group.id)
            gr.permissions.clear()
            try:
                for permission in datas:
                    try:
                        p = Permission.objects.get(codename=permission['codename'])
                        gr.permissions.add(p)
                    except:

                        content_type, created = ContentType.objects.get_or_create(
                            model='downloaded',
                        )
                        custom_permission, created = Permission.objects.get_or_create(content_type= content_type, codename=permission['codename'], defaults={'name': permission['name']})
                        gr.permissions.add(custom_permission)
            except:
                pass
            gr.save()

            for entidade in Entidade.objects.filter(tipo_entidade_id=id):
                for g in te.groups.all():
                    entidade.groups.add(g)

        te = {'id': te.id, 'nome': te.nome, 'alert_success': 'Permicoes baixadas para<br><b>'+te.nome+'</b>'}
        
        return Response(te , status=status.HTTP_200_OK)
    
    @action(
        detail=True,
        methods=['GET'],
    )
    def uploadPermission(self, request, id):
        te = TipoEntidade.objects.get(id=id)
        grupos ={}
        for group in te.groups.all():
            ref = db.reference('Premissions').child(te.nome)
            data = PermissionSerializer(group.permissions.all(), many=True).data
            grupos[group.name] = data
        ref.set(grupos)     
        te = {'id': te.id, 'nome': te.nome, 'alert_success': 'Permicoes actualizadas para<br><b>'+te.nome+'</b>'}
        
        return Response(te , status=status.HTTP_200_OK)

    @action(
        detail=True,
        methods=['PUT'],
    )
    def perfilPut(self, request, id):
        group = Group.objects.get(id=request.data['id'])
        group.name = request.data['name']
        group.save()
        
        perfil = {'id': group.id, 'name': group.name, 'alert_success': 'Perfil <b>'+ group.name + ' </b> actualizado com sucesso'}
        return Response(perfil, status.HTTP_201_CREATED)


    @action(detail=True, methods=['POST'],)
    def perfilPost(self, request, id):
        tipoentidade = TipoEntidade.objects.get(id=id)        
        group = Group.objects.create(name=request.data['name'])
        tipoentidade.groups.add(group)
        for entidade in Entidade.objects.filter(tipo_entidade_id=id):
            entidade.groups.add(group)

        perfil = {'id': group.id, 'name': group.name, 'alert_success': 'Perfil <b>'+ group.name + ' </b> criado com sucesso'}
        return Response(perfil, status.HTTP_201_CREATED)
    
    @action(detail=True, methods=['GET'],)
    def apps(self, request, id):
        tipoentidade = TipoEntidade.objects.get(id=id)        
        apps_ = []
        
        for app in apps.get_app_configs():
            apps_.append({'name': app.name, 'label': app.label, 'verbose': app.verbose_name })
            
        for app in dj_settings.INSTALLED_APPS:
            apps_.append(app)


        return Response(apps_, status.HTTP_200_OK)
    

    @action(detail=True,methods=['GET'],)
    def menus(self, request, *args, **kwargs):
        transformer = self.get_object()
        ALL_MENUS = {}
        MENUS =[]
        for app in apps.get_app_configs():
            sidebar = None
            module_name = app.name + ".sidebar"
            if importlib.util.find_spec(module_name):
                sidebar = importlib.import_module(module_name)
            else:       
                continue

            if sidebar:
                MENU = {}
                MENU["app"] = app.label
                MENU["menu"] = sidebar.MENU
                MENU["icon"] = sidebar.ICON
                MENU["role"] = sidebar.ROLE
                MENU["submenu"] = []
                for submenu in sidebar.SUBMENUS:
                    MENU["submenu"].append(submenu) 

                MENUS.append(MENU)

        return Response(MENUS, status.HTTP_200_OK)
    
    @action(
        detail=True,
        methods=['GET'],
    )
    def modelos(self, request, id):
        tipoentidade = TipoEntidade.objects.get(id=id)        
        modelos = []
        for modelo in tipoentidade.modelos.all():
           modelos.append({'id': modelo.id, 'model': modelo.model, 'app_label':  modelo.app_label })

        return Response(modelos, status.HTTP_200_OK)
    
    @action(
        detail=True,
        methods=['POST'],
    )
    def addModelo(self, request, id):
        tipoentidade = TipoEntidade.objects.get(id=id)        
        modelo = ContentType.objects.get(id=request.data['id'])
        tipoentidade.modelos.add(modelo)
        for entidade in Entidade.objects.filter(tipo_entidade_id=id):
            entidade.modelos.add(modelo)
            
        modelo = {'id': modelo.id, 'model': modelo.model, 'alert_success': 'App <b>'+ modelo.model + ' </b> criado com sucesso'}
        return Response(modelo, status.HTTP_201_CREATED)

    @action(
        detail=True,
        methods=['POST'],
    )
    def removeModelo(self, request, id):
        tipoentidade = TipoEntidade.objects.get(id=id)        
        modelo = ContentType.objects.get(id=request.data['id'])
        tipoentidade.modelos.remove(modelo)
        for entidade in Entidade.objects.filter(tipo_entidade_id=id):
            entidade.modelos.remove(modelo)

        modelo = {'id': modelo.id, 'model': modelo.model, 'alert_success': 'App <b>'+ modelo.model + ' </b> Removido com sucesso'}
        return Response(modelo, status.HTTP_201_CREATED)

    @action(
        detail=True,
        methods=['GET'],
    )
    def modulos(self, request, *args, **kwargs):
        transformer = self.get_object()
        EntMods = TipoEntidadeModulo.objects.filter(tipo_entidade = transformer.id)        
        modulos = [
            {'id': TiEntMo.modelo.id, 'nome': TiEntMo.modelo.nome}
            for TiEntMo in EntMods
        ]

        return Response(modulos, status=status.HTTP_200_OK)

class  EntidadeAPIView(viewsets.ModelViewSet):
    search_fields = ['id','nome']
    filter_backends = (filters.SearchFilter,)
    serializer_class = EntidadeSerializer
    queryset = Entidade.objects.all()


    def get_queryset(self, *args, **kwargs):
        return self.queryset.filter().order_by('-id')

    def retrieve(self, request, *args, **kwargs):
        try:
            transformer = self.get_object()
            paciente = EntidadeSerializer(transformer, context={'request': request})
            return Response(paciente.data, status=status.HTTP_200_OK)
        except Http404:
            pass
        return Response(status=status.HTTP_404_NOT_FOUND)


    def destroy(self, request, *args, **kwargs):
        try:
            instance = self.get_object()
            self.perform_destroy(instance)
        except Http404:
            pass
        return Response(status=status.HTTP_204_NO_CONTENT)

    def list(self, request, *args, **kwargs):
        self._paginator = None
        queryset = self.filter_queryset(self.get_queryset().filter().order_by('-id'))
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True, context={'request': request})
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True, context={'request': request})
        return Response(serializer.data, status=status.HTTP_200_OK)

    def update(self, request,  *args, **kwargs):

        transformer = self.get_object()

        entidade = EntidadeSerializer(transformer, data=request.data)
        if entidade.is_valid(raise_exception=True):
            entidade.save()
            return Response(entidade.data, status=status.HTTP_201_CREATED)
        else:
            return Response(entidade.errors, status=status.HTTP_400_BAD_REQUEST)



    def create(self, request, *args, **kwargs):
        
        tipo_entidade_id = request.headers.get('ET')
        entidade_id = request.headers.get('E')
        sucursal_id = request.headers.get('S')
        grupo_id = request.headers.get('G')
        if self.request.query_params.get('selfRegist') == 'self':
            request.data['tipo_entidade'] = tipo_entidade_id
            request.data['admin'] = request.user.id

        entidade = EntidadeGravarSerializer(data=request.data)

        if entidade.is_valid(raise_exception=True):
            entidadeSave = entidade.save()

            entidadeUser = EntidadeUser()
            entidadeUser.user = request.user
            entidadeUser.entidade = entidadeSave
            entidadeUser.save()

            tipoEntidade = TipoEntidade.objects.all().filter(id=entidadeSave.tipo_entidade.id).first()
            for group in tipoEntidade.groups.all():
                entidadeSave.groups.add(group)

            sucursal = Sucursal()
            sucursal.nome = entidadeSave.nome+' Sede'
            sucursal.entidade = entidadeSave
            sucursal.endereco = '...'
            sucursal.icon = '...'
            sucursal.label = '...'

            sucursal.save()

            sucursalUser = SucursalUser()
            sucursalUser.user = request.user
            sucursalUser.sucursal = sucursal
            sucursalUser.save()

            user = User.objects.all().filter(id=request.user.id).first()
            for group in tipoEntidade.groups.all():
                sucursal.groups.add(group)
                user.groups.add(group)

                sucursalUserGroup = SucursalUserGroup()
                sucursalUserGroup.group = group
                sucursalUserGroup.user = request.user
                sucursalUserGroup.sucursal = sucursal
                sucursalUserGroup.save()

            return Response(entidade.data, status=status.HTTP_201_CREATED)

        else:
            return Response(entidade.errors, status=status.HTTP_400_BAD_REQUEST)

    @action(
        detail=True,
        methods=['GET'],
    )
    def sucursals(self, request, *args, **kwargs):
        transformer = self.get_object()
        sucursals = Sucursal.objects.filter(entidade__id=transformer.id)
        suc = []
        for sucursal in sucursals:
            try:
                suc.append({'id': sucursal.id, 'nome': sucursal.nome, 'estado': sucursal.estado})
            except:
                pass
        return Response(suc)
    
    @action(
        detail=True,
        methods=['GET'],
    )
    def modelos(self, request, *args, **kwargs):
        transformer = self.get_object()
        entidade = Entidade.objects.get(id= transformer.id)        
        modelos = []
        for modelo in entidade.modelos.all():
           modelos.append({'id': modelo.id, 'model': modelo.model, 'app_label':  modelo.app_label })

        return Response(modelos, status.HTTP_200_OK)
    
    @action(
        detail=True,
        methods=['GET'],
    )
    def modulos(self, request, *args, **kwargs):
        transformer = self.get_object()
        EntMods = EntidadeModulo.objects.filter(entidade= transformer.id)        
        modulos = [
            {'id': EntMo.modelo.id, 'nome': EntMo.modelo.nome}
            for EntMo in EntMods
        ]

        return Response(modulos, status=status.HTTP_200_OK)


    
    @action(detail=True, methods=['POST'],)
    def addModelo(self, request, *args, **kwargs):
        transformer = self.get_object()
        entidade = Entidade.objects.get(id=transformer.id)        
        modelo = ContentType.objects.get(id=request.data['id'])
        entidade.modelos.add(modelo)

        modelo = {'id': modelo.id, 'model': modelo.model, 'alert_info': 'App <b>'+ modelo.app_label + ' </b> criado com sucesso'}
        return Response(modelo, status.HTTP_201_CREATED)

    @action(
        detail=True,
        methods=['POST'],
    )
    def removeModelo(self, request, *args, **kwargs):
        transformer = self.get_object()
        entidade = Entidade.objects.get(id=transformer.id)        
        modelo = ContentType.objects.get(id=request.data['id'])
        entidade.modelos.remove(modelo)
        
        modelo = {'id': modelo.id, 'model': modelo.model, 'alert_info': 'App <b>'+ modelo.app_label + ' </b> Removido com sucesso'}
        return Response(modelo, status.HTTP_201_CREATED)


    @action(
        detail=True,
        methods=['GET'],
    )
    def perfils(self, request, *args, **kwargs):
        transformer = self.get_object()
        entidade = Entidade.objects.get(id=transformer.id)

        perfils = []
        for group in entidade.groups.all():
            perfils.append({'id': group.id, 'name': group.name})
            
        perfils = sorted(perfils, key=lambda arguments : arguments['name'])

        return Response(perfils, status.HTTP_200_OK)    

    @action(
        detail=True,
        methods=['GET'],
    )
    def usuarios(self, request, *args, **kwargs):
        transformer = self.get_object()
        search = self.request.query_params.get('search')
        entidadeUsers = EntidadeUser.objects.filter(entidade__id=transformer.id, user__username__icontains = search, is_deleted = False  ).order_by('-user__username')
        page = self.paginate_queryset(entidadeUsers)
        serializer = EntidadeUserSerializer(page, many=True)
        return self.get_paginated_response(serializer.data)


    @action(
        detail=True,
        methods=['POST'],
    )
    def addUser(self, request, *args, **kwargs):
        transformer = self.get_object()
        entidadeUser = EntidadeUser()
        entidadeUser.user = User.objects.get(id = request.data['user'])
        entidadeUser.entidade = Entidade.objects.get(id = transformer.id)
        
        if len(EntidadeUser.objects.filter(entidade__id=transformer.id, user = entidadeUser.user, is_deleted = False  )) < 1 :
            data = entidadeUser.save()
            adda = {"alert_seccess": "O usuario " +  entidadeUser.user.username + " adicionado com sucesso!"}
            return Response (adda, status=status.HTTP_201_CREATED)
            
        
        adda = {"alert_seccess": "O usuario " +  entidadeUser.user.username + " ja existe !"}
        return Response(adda, status.HTTP_201_CREATED)
    

    @action(
        detail=True,
        methods=['DELETE'],
    )
    def removeUser(self, request, *args, **kwargs):
        transformer = self.get_object()
        try:
            entidadeUser = EntidadeUser.objects.filter(entidade__id=transformer.id, is_deleted = False, user__id = request.query_params.get('user') ).first()
            entidadeUser.is_deleted = True
            entidadeUser.save()
            return Response(status=status.HTTP_204_NO_CONTENT)
        except:
             return Response("entidade.errors", status=status.HTTP_400_BAD_REQUEST)



    @action(
        detail=True,
        methods=['GET'],
    )
    def getFornecedors(self, request, *args, **kwargs):
        transformer = self.get_object()
        fornecedores = Fornecedor.objects.filter(entidade=transformer)
        fornecedores =FornecedorSerializer(fornecedores, many=True)
        return Response(fornecedores.data, status.HTTP_200_OK)


    @action(
        detail=True,
        methods=['POST'],
    )
    def logoPost(self, request, *args, **kwargs):

        transformer = self.get_object()
        entidade = Entidade.objects.get(id=transformer.id)
       
        request.data['entidade'] = str(entidade.id)
        uploaded_file = request.FILES['ficheiro']

        if DiscManegar.freeSpace(entidade.id, request.FILES['ficheiro']):
            resposta = {'alert_error': 'Nao e possivel fazer upload de ficheiro<br><b>Contacte o adminstrador</b>'}
            return Response(resposta , status=status.HTTP_400_BAD_REQUEST)
        


        try:
            fcr = Ficheiros.objects.get(entidade=entidade, funcionalidade='Logo')
            fcr.delete()
            DiscManegar.recoverSpace(entidade.id, fcr)
        except:
            print('Nao apgaou')
    

        request.data['size'] = uploaded_file.size
        request.data['modelo'] = 'Entidade'
        request.data['estado'] = 'Activo'
        request.data['funcionalidade'] = 'Logo'

        ficheiro = FicheiroGravarSerializer(data=request.data)
        if ficheiro.is_valid(raise_exception=True):
            ficheiro.save()
            ficheiro = FicheiroSerializer(Ficheiros.objects.get(id=ficheiro.data['id']))
            DiscManegar.updateSpace(entidade.id, request.FILES['ficheiro'])
            return Response(ficheiro.data, status=status.HTTP_201_CREATED)
        else:
            return Response(ficheiro.errors, status=status.HTTP_400_BAD_REQUEST)


    @action(
        detail=True,
        methods=['GET'],
    )
    def qr(self, request, pk):
        id = pk
        var_qr = {}
        origin = request.headers['Origin']
        LANGUAGE_CODE = 'pt-pt'

        TIME_ZONE = 'UTC'
        settings.LANGUAGE_CODE = 'pt-pt'
        # django.setup()
        print(settings.LANGUAGE_CODE)

        root = settings.MEDIA_ROOT
        lingua = self.request.query_params.get('lang')

        ean = barcode.get('code128', id, writer=ImageWriter())
        filename = ean.save(str(root) +'/' + str(random.random()) + 'qr' + str(random.random()))

        file = Image.open(str(filename))
        file = open(str(filename), 'rb').read()


        blob_barcode = base64.b64encode((file))
        if os.path.exists(filename):
            os.remove(filename)


        qr = qrcode.QRCode(box_size=2)
        qr.add_data(str('var_qr'))
        qr.make()
        img_qr = qr.make_image()
        # img_qr.
        img = img_qr.get_image()

        name = str(root) +'/' + str(random.random()) + 'qr' + str(random.random()) + '.png'
        img_qr.save(name)
        file = Image.open(str(name))
        file = open(str(name), 'rb').read()
        blob = base64.b64encode(bytes(file))
        if os.path.exists(name):
            os.remove(name)


        template_path = 'core/entidade/qr_pdf.html'

        entidade = Entidade.objects.get(id=id)
 
        entidade = EntidadeSerializer(entidade)

        ficheiro  = Ficheiros.objects.get(entidade = id, funcionalidade = 'Logo')

        logo_name = ficheiro.ficheiro.path
        try:
            file = open(logo_name, 'rb').read()
            logo = base64.b64encode(file)
        except:
            logo = ''

        
        url = origin + '/#/?e=' + entidade.data['id'] + '&q=1' 
        var_qr['entidade'] = entidade.data['nome']
        for key, value in var_qr.items():
            url = url + '&' + key + '=' + value
        qr = qrcode.QRCode(box_size=2)
        qr.add_data(str(url))
        qr.make()
        img_qr = qr.make_image()
    

        name = str(root) +'/' + str(random.random()) + 'qr' + str(random.random()) + '.png'
        img_qr.save(name)
        file = Image.open(str(name))
        file = open(str(name), 'rb').read()
        qr_to_scan = base64.b64encode(bytes(file))
        if os.path.exists(name):
            os.remove(name)
        context = {
            'qr': blob,
            'qr_to_scan': qr_to_scan,
            'barcode': blob_barcode, 
            'entidade': entidade.data,
            'logo':logo,
            'titulo': Translate.tdc(lingua, 'QR'),
            'nome': Translate.tdc(lingua, 'Entidade'),
            'de': Translate.tdc(lingua, 'de'),
            'morada': Translate.tdc(lingua, 'Morada'),
            'pagina': Translate.tdc(lingua, 'Pagina')
        }
        # Create a Django response object, and specify content_type as pdf
        response = HttpResponse(content_type='application/pdf')
        # if you need to download
        # response['Content-Disposition'] = 'attachment; filename="report.pdf"'
        response['Content-Disposition'] = 'filename="report.pdf"'
        # find the template and render it.
        template = get_template(template_path)
        html = template.render(context)

        # create a pdf                  link_callback=link_callback
        pisa_status = pisa.CreatePDF(
            html, dest=response)
        # if error then show some funy view
        if pisa_status.err:
            return HttpResponse('We had some errors <pre>' + html + '</pre>')
        return response


    @action(
        detail=True,
        methods=['PUT'],
    )
    def perfilPut(self, request, pk):
        group = Group.objects.get(id=request.data['id'])
        group.name = request.data['name']
        group.save()
        
        perfil = {'id': group.id, 'name': group.name, 'alert_success': 'Perfil <b>'+ group.name + ' </b> actualizado com sucesso'}
        return Response(perfil, status.HTTP_201_CREATED)


    @action(
        detail=True,
        methods=['POST'],
    )
    def perfilPost(self, request, pk):      
        group = Group.objects.create(name=request.data['name'])
        entidade = Entidade.objects.get(id=pk)
        entidade.groups.add(group)

        perfil = {'id': group.id, 'name': group.name, 'alert_success': 'Perfil <b>'+ group.name + ' </b> criado com sucesso'}
        return Response(perfil, status.HTTP_201_CREATED)




class  SucursalAPIView(viewsets.ModelViewSet):
    #permission_classes = (permissions.IsAuthenticated)
    search_fields = ['id','nome']
    filter_backends = (filters.SearchFilter,)
    
    serializer_class = SucursalSerializer
    queryset = Sucursal.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        return self.queryset.filter().order_by('-id')

    @action(
        detail=True,
        methods=['GET'],
    )
    def grupos(self, request, id):
        sucursalUserGroups = SucursalUserGroup.objects.all().filter(sucursal__id=id, user__id=request.user.id)
        suc = []
        for sucursalUserGroup in sucursalUserGroups:
            suc.append({'id': sucursalUserGroup.group.id, 'name': sucursalUserGroup.group.name})

        return Response(suc)

    @action(
        detail=True,
        methods=['GET'],
    )
    def Url(self, request, id):
        sucursal = Sucursal.objects.get(id=id)
        entidade = Entidade.objects.get(id=sucursal.entidade.id)
        return Response(str(entidade.tipo_entidade.id)+'/'+str(sucursal.entidade.id)+'/'+str(sucursal.id))
    
    

    @action(
        detail=True,
        methods=['GET'],
    )
    def getCapasSite(self, request, id):
        ficheiros = Ficheiros.objects.filter(sucursal__id=id, funcionalidade='CapaSite')
        ficheiros = FicheiroSerializer(ficheiros, many=True)
        data = (ficheiros.data)
        return Response(data, status=status.HTTP_200_OK)
    

    @action(
        detail=True,
        methods=['POST'],
    )
    def postCapasSite(self, request, id):
        sucursal = Sucursal.objects.get(id=id)

        request.data['entidade'] = str(sucursal.entidade.id)
        request.data['sucursal'] = str(sucursal.id)
        uploaded_file = request.FILES['ficheiro']

        if DiscManegar.freeSpace(sucursal.entidade.id, request.FILES['ficheiro']):
            resposta = {'alert_error': 'Nao e possivel fazer upload de ficheiro<br><b>Contacte o adminstrador</b>'}
            return Response(resposta , status=status.HTTP_400_BAD_REQUEST)

        request.data['size'] = uploaded_file.size
        request.data['modelo'] = 'sucursal'
        request.data['estado'] = 'Activo'
        request.data['funcionalidade'] = 'CapaSite'

        ficheiro = FicheiroGravarSerializer(data=request.data)
        if ficheiro.is_valid(raise_exception=True):
            ficheiro.save()
            ficheiro = FicheiroSerializer(Ficheiros.objects.get(id=ficheiro.data['id']))
            DiscManegar.updateSpace(sucursal.entidade.id, request.FILES['ficheiro'])
            return Response(ficheiro.data, status=status.HTTP_201_CREATED)
        else:
            return Response(ficheiro.errors, status=status.HTTP_400_BAD_REQUEST)
        
    @action(
        detail=True,
        methods=['GET'],
    )
    def getStocks(self, request, *args, **kwargs):
        transformer = self.get_object()
        stock = Stock.objects.filter(sucursal=transformer)
        stocks =StockSerializer(stock, many=True)
        return Response(stocks.data, status.HTTP_200_OK)
   

    

    @action(
        detail=True,
        methods=['GET'],
    )
    def qr(self, request, id):
        var_qr = {}
        origin = request.headers['Origin']
        LANGUAGE_CODE = 'pt-pt'

        TIME_ZONE = 'UTC'
        settings.LANGUAGE_CODE = 'pt-pt'
        # django.setup()
        print(settings.LANGUAGE_CODE)

        root = settings.MEDIA_ROOT
        lingua = self.request.query_params.get('lang')

        ean = barcode.get('code128', id, writer=ImageWriter())
        filename = ean.save(str(root) +'/' + str(random.random()) + 'qr' + str(random.random()))

        file = Image.open(str(filename))
        file = open(str(filename), 'rb').read()


        blob_barcode = base64.b64encode((file))
        if os.path.exists(filename):
            os.remove(filename)


        qr = qrcode.QRCode(box_size=2)
        qr.add_data(str('var_qr'))
        qr.make()
        img_qr = qr.make_image()
        # img_qr.
        img = img_qr.get_image()

        name = str(root) +'/' + str(random.random()) + 'qr' + str(random.random()) + '.png'
        img_qr.save(name)
        file = Image.open(str(name))
        file = open(str(name), 'rb').read()
        blob = base64.b64encode(bytes(file))
        if os.path.exists(name):
            os.remove(name)

        pk = id


        template_path = 'core/sucursal/qr_pdf.html'

        sucursal = Sucursal.objects.get(id=pk)
        sucursal1 = sucursal

    

        entidade = EntidadeSerializer(sucursal.entidade)

        sucursal = SucursalSerializer(sucursal)
        ficheiro  = Ficheiros.objects.get(entidade = sucursal1.entidade.id, funcionalidade = 'Logo')
        logo_name = ficheiro.ficheiro.path
        try:
            file = open(logo_name, 'rb').read()
            logo = base64.b64encode(bytes(file))
        except Exception as e:
            logo = logo_name.split('.')[-1]
        print(logo, logo_name, ficheiro)
        
        url = origin + '/#/?s=' + sucursal.data['id'] + '&q=1' 
        var_qr['entidade'] = entidade.data['nome']
        var_qr['sucursal'] = sucursal.data['nome']
        for key, value in var_qr.items():
            url = url + '&' + key + '=' + value
        # print(url)
        qr = qrcode.QRCode(box_size=2)
        qr.add_data(str(url))
        qr.make()
        img_qr = qr.make_image()
    

        name = str(root) +'/' + str(random.random()) + 'qr' + str(random.random()) + '.png'
        img_qr.save(name)
        file = Image.open(str(name))
        file = open(str(name), 'rb').read()
        qr_to_scan = base64.b64encode(bytes(file))


        if os.path.exists(name):
            os.remove(name)
        context = {
            'qr': blob,
            'qr_to_scan': qr_to_scan,
            'barcode': blob_barcode, 
            'entidade': entidade.data,
            'sucursal': sucursal.data,
            'logo':logo,
            'titulo': Translate.tdc(lingua, 'QR'),
            'nome': Translate.tdc(lingua, 'Sucursal'),
            'de': Translate.tdc(lingua, 'de'),
            'morada': Translate.tdc(lingua, 'Morada'),
            'pagina': Translate.tdc(lingua, 'Pagina')
        }
        # Create a Django response object, and specify content_type as pdf
        response = HttpResponse(content_type='application/pdf')
        # if you need to download
        # response['Content-Disposition'] = 'attachment; filename="report.pdf"'
        response['Content-Disposition'] = 'filename="report.pdf"'
        # find the template and render it.
        template = get_template(template_path)
        html = template.render(context)

        

        # create a pdf                  link_callback=link_callback
        pisa_status = pisa.CreatePDF(
            html, dest=response)
        # if error then show some funy view
        if pisa_status.err:
            return HttpResponse('We had some errors <pre>' + html + '</pre>')
        return Response(context, status=status.HTTP_200_OK)
        return response

class  GrupoAPIView(viewsets.ModelViewSet):
    #permission_classes = (permissions.IsAuthenticated)
    # permission_classes = (permissions.IsAuthenticated, CoreAccessPermission)
    # permissions_rules = Permission.objects.filter(content_type__app_label='clinica',
    #                                               content_type__model='itempedidoexamemedico')
    paginator = None
    search_fields = ['id','name']
    filter_backends = (filters.SearchFilter,)
    serializer_class = GrupoSerializer
    queryset = Group.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        return self.queryset.filter().order_by('-codename')
        

    def retrieve(self, request, id, *args, **kwargs):
        try:
            request.query_params['permissions']
            grupo = Group.objects.get(id=id)
            permissions = grupo.permissions.annotate(content_type_model= F('content_type__model'), content_type_app= F('content_type__app_label'))
    
            per = []
            for permission in permissions:
                per.append({'id':permission.id, 'name':permission.name , 'codename':permission.codename, 'content_type':permission.content_type.id, 'content_type_model': permission.content_type_model, 'content_type_app': permission.content_type_app})
            print(per)
            return Response(per)
        except:
           pass

        try:

            transformer = self.get_object()
            grupo = GrupoSerializer(transformer)
            return Response(grupo.data, status=status.HTTP_200_OK)
        except Http404:
            pass
        return Response( status=status.HTTP_404_NOT_FOUND)
    
    def update(self, request,id,  *args, **kwargs):
        g = Group.objects.get(id=id)
        g.name = request.data['name']
        g.save()
        add={'alert_success': '%-'+ request.data['name'] +'-% foi actualizado com sucesso'}
        data = json.loads(json.dumps({'id':g.id}))
        data.update(add)

        return Response(data, status=status.HTTP_202_ACCEPTED)

    def destroy(self, request, id,  *args, **kwargs):
        g = Group.objects.get(id=id)
        nome = g.name
        g.delete()
        add={'alert_success': '<b>'+ nome +'</b> foi apagado com sucesso'}
        # return Response(add,status=status.HTTP_204_NO_CONTENT)
        return Response(add,status=status.HTTP_202_ACCEPTED)


    @action(
        detail=True,
        methods=['POST'],
    )
    def addPermission(self, request, id):
        grupo = Group.objects.get(id=id)

        content_type, created = ContentType.objects.get_or_create(
            model='uploadloaded',
        )
        custom_permission, created = Permission.objects.get_or_create(content_type= content_type, codename=request.data['codename'], defaults={'name': request.data['name']})
        grupo.permissions.add(custom_permission)
        permission = Permission.objects.get(id=custom_permission.id)

        per= {'id': permission.id, 'nome': permission.codename, 'nomeseparado':permission.name, 'alert_success': 'Permicao <b>' +permission.name + '</b> foi Adicionado consucesso'}
        return Response(per,status.HTTP_201_CREATED )

    


class  PermissionAPIView(viewsets.ModelViewSet):
 
    search_fields = ['id','name']
    filter_backends = (filters.SearchFilter,)
    serializer_class = PermissionSerializer
    queryset = Permission.objects.annotate(content_type_model= F('content_type__model'), content_type_app= F('content_type__app_label'))
    lookup_field = "id"
    paginator = None


    def get_queryset(self):
        return self.queryset.filter().order_by('codename')

    def list(self, request, *args, **kwargs):
        self._paginator = None
        queryset = self.filter_queryset(self.get_queryset().filter().order_by('content_type__app_label', 'content_type__model', 'codename'))
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)
    
    @action(
        detail=True,
        methods=['POST'],
    )
    def addToGroup(self, request, id):
        grupo = Group.objects.get(id=request.data['id'])
        permission = Permission.objects.get(id=id)
        grupo.permissions.add(permission)
    
        per= {'id': permission.id, 'nome': permission.codename, 'nomeseparado':permission.name, 'alert_success': 'Permicao <b>' +permission.name + '</b> foi Adicionado consucesso'}
        return Response(per,status.HTTP_201_CREATED )


    @action(
        detail=True,
        methods=['POST'],
    )
    def removeFromGroup(self, request, id):
        # print(id, request.data) 
        grupo = Group.objects.get(id=request.data['id'])
        permission = Permission.objects.get(id=id)
        grupo.permissions.remove(permission)
    
        per= {'id': permission.id, 'nome': permission.codename, 'nomeseparado':permission.name, 'alert_info': 'Permicao <b>' +permission.name + '</b> foi removido consucesso'}
        return Response(per,status.HTTP_201_CREATED )


    @action(
        detail=True,
        methods=['POST'],
    )
    def addToUser(self, request, id):
        user = User.objects.get(id=request.data['user'])
        group = Group.objects.get(id=id)
        SucursalUserGroup.objects.create(sucursal_id=request.data['sucursal'], user=user, group=group)
        user.groups.add(group)
        per= {'alert_success': 'Perfil <b>' +group.name + '</b> foi Adicionado consucesso'}
        return Response(per,status.HTTP_201_CREATED )


    @action(
        detail=True,
        methods=['POST'],
    )
    def removeFromUser(self, request, id):
        user = User.objects.get(id=request.data['user'])
        group = Group.objects.get(id=id)
        SucursalUserGroup.objects.get(sucursal_id=request.data['sucursal'], user=user, group=group).delete()
        per= {'alert_success': 'Perfil <b>' +group.name + '</b> foi Removido consucesso'}
        return Response(per,status.HTTP_200_OK )


